/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.impl.AbstractResourceHome;
import org.apache.ws.resource.lifetime.faults.ResourceUnknownFaultException;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.ws.util.i18n.Messages;

import javax.xml.rpc.JAXRPCException;

/**
 * LOG-DONE
 * @author Ian P. Springer
 */
public abstract class AbstractPortType
{

   private static final Log LOG = LogFactory.getLog( AbstractPortType.class );
   public static final Messages MSG = MessagesImpl.getInstance();

   private ResourceContext m_resourceContext;
   private ResourceHome    m_resourceHome;
   private ResourceKey     m_resourceKey;
   private Resource        m_resource;

   /**
    * Creates a new {@link AbstractPortType} object.
    *
    * @param resourceContext DOCUMENT_ME
    */
   public AbstractPortType( ResourceContext resourceContext )
   {
      m_resourceContext    = resourceContext;
      m_resourceHome       = null;
      try
      {
         m_resourceHome = (AbstractResourceHome) getResourceContext(  ).getResourceHome(  );
      }
      catch ( ResourceContextException rce )
      {
         throw new JAXRPCException( rce );
      }

      try
      {
         m_resourceKey = getResourceContext(  ).getResourceKey(  );
      }
      catch ( ResourceContextException rce )
      {
         throw new JAXRPCException( rce );
      }

      LOG.debug( MSG.getMessage(Keys.PORTTYPE_RECIEVED_REQUEST, m_resourceContext.getServiceName(), String.valueOf(m_resourceKey)) );
      try
      {
         m_resource = ( (ResourceHome) m_resourceHome ).getInstance( m_resourceContext );
      }
      catch ( ResourceUnknownException rce )
      {
         throw new org.apache.ws.resource.properties.faults.ResourceUnknownFaultException(getNamespaceSet(),rce);
      }
      catch ( ResourceException re )
      {
         throw new JAXRPCException( re );
      }
      catch ( ResourceContextException rce )
      {
         rce.printStackTrace(  );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Resource getResource(  )
   {
      return m_resource;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public ResourceContext getResourceContext(  )
   {
      return m_resourceContext;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public ResourceHome getResourceHome(  )
   {
      return m_resourceHome;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public ResourceKey getResourceKey(  )
   {
      return m_resourceKey;
   }

    protected abstract NamespaceVersionHolder getNamespaceSet();
}