/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource;


/**
 * Defines callback operations for persistence operations. These operations will be invoked when used with {@link
 * org.apache.ws.resource.impl.ResourceHomeImpl}. Currently, the <code>ResourceHomeImpl</code> will only invoke the
 * {@link #load(ResourceKey) load()} function automatically. The resource implementation itself is responsible for
 * calling {@link #store() store()} function to synchronize its state on disk.
 */
public interface PersistenceCallback
{
   /**
    * Loads the resource state.
    *
    * @throws ResourceUnknownException     if no resource state exists for the specified key.
    * @throws InvalidResourceKeyException if the resource key is invalid.
    * @throws ResourceException           if the load operation fails for any other reason.
    */
   void load( ResourceKey key )
   throws ResourceException, 
          ResourceUnknownException,
          InvalidResourceKeyException;

   /**
    * Saves the resource state.
    *
    * @throws ResourceException if the store operation fails.
    */
   void store(  )
   throws ResourceException;
}