/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource;


/**
 * Defines a callback operation that is invoked whenever a resource is removed from {@link ResourceHome ResourceHome}.
 * This is an optional callback and the resource does need to implement this function if it does not wish to be notified
 * when the resource is removed.
 */
public interface RemoveCallback
{
   /**
    * Notifies that the resource was removed. This function must not be called directly on the resource object. Only
    * {@link ResourceHome#remove(ResourceKey) ResourceHome.remove()} is allowed to call that method during the remove
    * operation.
    *
    * @throws ResourceException if the remove operation fails.
    */
   void remove(  )
   throws ResourceException;
}