/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource;



/**
 * A marker interface for a resource. All resource objects must implement this interface.
 */
public interface Resource
{

    /**
    * @param id
    *
    * @throws IllegalArgumentException if the specified ID is not the of the correct type for this resource
    * @throws IllegalStateException    if this resource's ID has already been set
    */
   void setID( Object id );

   /**
    * Returns the unique id of the resource. In most cases, this value should match value returned by {@link
    * ResourceKey#getValue}.
    *
    * @return the id of the resource.
    */
   Object getID(  );

      /**
    * The WS-RF runtime system ends the lifecycle of a {@link Resource}
    * object by invoking this method.
    */
   void destroy();

   /**
    * The WS-RF runtime system initializes a {@link Resource} object by
    * invoking this method.
    *
    */
   void init();
}