/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource;

import org.apache.commons.lang.SerializationException;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import java.io.Serializable;

/**
 * A basic representation of a resource key. A resource key is composed of name and the actual resource key value.
 */
public interface ResourceKey
   extends Serializable
{
   /**
    * The name of the key.
    *
    * @return the name of the key. Cannot be null.
    */
   QName getName(  );

   /**
    * The actual key value.
    *
    * @return the key value. Cannot be null.
    */
   Object getValue(  );

   /**
    * Converts the resource key into a {@link SOAPElement SOAPElement}. The element name and namespace must match the
    * name of the key.
    *
    * @return the resource key as a <code>SOAPElement</code>
    *
    * @throws SerializationException in case the conversion fails.
    */
   SOAPElement toSOAPElement(  )
   throws SerializationException;

   /**
    * Returns the class representation of the ResourceKey implementation.
    *
    * @return The Class of the key type.
    */
   Class getType();

}