/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.faults;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.util.i18n.Messages;
import org.apache.xmlbeans.XmlObject;

import java.net.URI;
import java.util.Arrays;

/**
 * The WS-BaseFaults ErrorCode type.
 *
 * @author Ian P. Springer
 */
public class ErrorCode
{
   public static final Messages MSG = MessagesImpl.getInstance();
   private URI           m_dialect; // in the XML type, this is a required attribute on the top-level ErrorCode element
   private XmlObject[]   m_content;

   /**
    * Creates a new ErrorCode with the specified dialect.
    *
    * @param dialect DOCUMENT_ME
    */
   public ErrorCode( URI dialect )
   {
      m_dialect = dialect;
   }

   /**
    * DOCUMENT_ME
    *
    * @param content DOCUMENT_ME
    */
   public void setContent( XmlObject[] content )
   {
      m_content = content;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public XmlObject[] getContent(  )
   {
      return m_content;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public URI getDialect(  )
   {
      return m_dialect;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String toString(  )
   {
      return new ToStringBuilder( this ).append( MSG.getMessage(Keys.DIALECT), m_dialect )
                                        .append( MSG.getMessage(Keys.CONTENT),
                                                 Arrays.asList( m_content ) ).toString(  );
   }
    
}