/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.faults;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.Soap1_1Constants;
import org.apache.ws.resource.NamespaceConstants;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.ws.util.i18n.Messages;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;

import javax.xml.namespace.QName;

/**
 * A fault that is thrown when the expected resourceKey header is not found in a SOAP request.
 *
 * @author Ian P. Springer
 */
public class ResoureKeyHeaderNotFoundFaultException
        extends FaultException
{
    private static final Messages MSG = MessagesImpl.getInstance();
    private static final Log LOG = LogFactory.getLog( ResoureKeyHeaderNotFoundFaultException.class );
    /**
     * DOCUMENT_ME
     */
    private static final String FAULT_LOCAL_NAME = "ResoureKeyHeaderNotFoundFault";

    /**
     * Creates a new {@link ResoureKeyHeaderNotFoundFaultException} object.
     *
     * @param expectedHeaderName DOCUMENT_ME
     */
    public ResoureKeyHeaderNotFoundFaultException( QName expectedHeaderName )
    {
        super( Soap1_1Constants.FAULT_CLIENT, null );
        try
        {
            XmlObject xBean = XmlObject.Factory.parse(
                    "<" + FAULT_LOCAL_NAME + " xmlns=\"" + NamespaceConstants.NSURI_APACHE_WS_RESOURCE + "\" />" );
            XmlBeanUtils.setValue( xBean,
                    MSG.getMessage( Keys.EXPECTED_ADDRESSING_HEADER, String.valueOf( expectedHeaderName ) ) );
            addDetailEntry( xBean );
        }
        catch ( XmlException xe )
        {
            LOG.error( xe );
        }
    }

}