/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.handler;

import org.apache.ws.resource.ResourceContext;

/**
 * Interface to be implemented by the Service implementation, either Abstract
 * or concrete.  Provides methods for initialization, obtaining the ResourceContext
 * and getting the SoapMethodNameMap use for mapping incoming request QName's to
 * method name for a given object.
 *
 * @author Sal Campana
 */
public interface WsrfService
{
   /**
    * Returns the SoapMethodNameMap for the Service, to determine
    * which method to invoke for an incoming request.
    *
    * @return SoapMethodNameMap
    */
   public SoapMethodNameMap getMethodNameMap(  );

   /**
    * Returns the ResourceContext for the given Service.
    *
    * @return ResourceContext
    */
   public ResourceContext getResourceContext(  );

   /**
    * Initialization method.
    */
   public void init(  );
}