/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.handler.axis;

import org.apache.axis.AxisFault;
import org.apache.axis.handlers.BasicHandler;

import javax.xml.rpc.handler.Handler;

/**
 * LOG-DONE
 * @author Ian Springer
 */
public class GenericAxisHandler
   extends BasicHandler
{
   private Handler jaxRpcHandler;

   /**
    * Creates a new {@link GenericAxisHandler} object.
    *
    * @param handler DOCUMENT_ME
    */
   public GenericAxisHandler( Handler handler )
   {
      jaxRpcHandler = handler;
   }

   /**
    * Impl of {@link org.apache.axis.Handler#invoke(org.apache.axis.MessageContext)} that delegates to either JAX-RPC
    * {@link Handler#handleRequest(javax.xml.rpc.handler.MessageContext)} or JAX-RPC {@link
    * Handler#handleResponse(javax.xml.rpc.handler.MessageContext)}, depending upon whether the handler chain pivot has
    * been passed yet or not.
    */
   public void invoke( org.apache.axis.MessageContext msgContext )
   throws AxisFault
   {
      if ( !msgContext.getPastPivot(  ) )
      {
         jaxRpcHandler.handleRequest( msgContext );
      }
      else
      {
         jaxRpcHandler.handleResponse( msgContext );
      }
   }

   /**
    * Impl of {@link org.apache.axis.Handler#onFault(org.apache.axis.MessageContext)} that delegates to JAX-RPC {@link
    * Handler#handleFault(javax.xml.rpc.handler.MessageContext)}.
    */
   public void onFault( org.apache.axis.MessageContext msgContext )
   {
      jaxRpcHandler.handleFault( msgContext );
   }
}