/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.handler.axis;

import org.apache.axis.Constants;
import org.apache.axis.message.MessageElement;
import org.apache.commons.lang.SerializationException;
import org.w3c.dom.Element;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import java.io.Writer;

/**
 * Converts Java Objects to DOM Elements and SOAP Elements. The objects must be compliant with the Axis Bean model, i.e.
 * generated using the WSDL2Java tool from an XML Schema definition or must be of simple type.
 */
public class ObjectSerializer
{
   /**
    * DOCUMENT_ME
    *
    * @param writer DOCUMENT_ME
    * @param obj    DOCUMENT_ME
    * @param name   DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public static void serialize( Writer writer,
                                 Object obj,
                                 QName  name )
   throws SerializationException
   {
      SOAPElement soapElement = ObjectSerializer.toSOAPElement( obj, name );
      try
      {
         AnyHelper.write( writer, (MessageElement) soapElement );
      }
      catch ( Exception e )
      {
         throw new SerializationException( "genericSerializationError", e );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public static Element toElement( Object obj )
   throws SerializationException
   {
      return toElement( obj, null, false );
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj  DOCUMENT_ME
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public static Element toElement( Object obj,
                                    QName  name )
   throws SerializationException
   {
      return toElement( obj, name, false );
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj      DOCUMENT_ME
    * @param name     DOCUMENT_ME
    * @param nillable DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public static Element toElement( Object  obj,
                                    QName   name,
                                    boolean nillable )
   throws SerializationException
   {
      if ( obj instanceof MessageElement )
      {
         MessageElement messageElement = (MessageElement) obj;
         if ( ( name == null ) || name.equals( messageElement.getQName(  ) ) )
         {
            Element element = null;
            try
            {
               element = AnyHelper.toElement( messageElement );
            }
            catch ( Exception e )
            {
               throw new SerializationException( "genericSerializationError", e );
            }

            return element;
         }
         else
         {
            throw new SerializationException( "notImplemented" );
         }
      }
      else if ( obj instanceof Element )
      {
         Element element = (Element) obj;
         if ( ( name == null )
              || ( name.getLocalPart(  ).equals( element.getLocalName(  ) )
              && name.getNamespaceURI(  ).equals( element.getNamespaceURI(  ) ) ) )
         {
            return element;
         }
         else
         {
            throw new SerializationException( "notImplemented" );
         }
      }

      MessageElement messageElement = (MessageElement) toSOAPElement( obj, name, nillable );
      try
      {
         return AnyHelper.toElement( messageElement );
      }
      catch ( Exception e )
      {
         throw new SerializationException( "genericSerializationError", e );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public static SOAPElement toSOAPElement( Object obj )
   throws SerializationException
   {
      return toSOAPElement( obj, null, false );
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj  DOCUMENT_ME
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public static SOAPElement toSOAPElement( Object obj,
                                            QName  name )
   throws SerializationException
   {
      return toSOAPElement( obj, name, false );
   }

   /**
    * Populates a SOAPElement with an arbitrary object. The object will get wrapped inside of an element named after the
    * qname parameter.
    *
    * @param obj  object to be serialized in the any element
    * @param name name of element the value should be wrapped inside
    *
    * @return content of any element as a SOAPElement
    *
    * @throws SerializationException if the object cannot be put in a MessageElement
    */
   public static SOAPElement toSOAPElement( Object  obj,
                                            QName   name,
                                            boolean nillable )
   throws SerializationException
   {
      if ( obj instanceof MessageElement )
      {
         MessageElement element = (MessageElement) obj;
         if ( ( name == null ) || name.equals( element.getQName(  ) ) )
         {
            return element;
         }
         else
         {
            throw new SerializationException( "notImplemented" );
         }
      }
      else if ( obj instanceof Element )
      {
         Element element = (Element) obj;
         if ( ( name == null )
              || ( name.getLocalPart(  ).equals( element.getLocalName(  ) )
              && name.getNamespaceURI(  ).equals( element.getNamespaceURI(  ) ) ) )
         {
            return new MessageElement( (Element) obj );
         }
         else
         {
            throw new SerializationException( "notImplemented" );
         }
      }

      if ( name == null )
      {
         throw new IllegalArgumentException( "nullArgument:name" );
      }

      MessageElement messageElement = new MessageElement(  );
      messageElement.setQName( name );
      try
      {
         messageElement.setObjectValue( obj );
      }
      catch ( Exception e )
      {
         throw new SerializationException( "genericSerializationError", e );
      }

      if ( ( obj == null ) && nillable )
      {
         try
         {
            messageElement.addAttribute( Constants.NS_PREFIX_SCHEMA_XSI, Constants.URI_DEFAULT_SCHEMA_XSI, "nil",
                                         "true" );
         }
         catch ( Exception e )
         {
            throw new SerializationException( "genericSerializationError", e );
         }
      }

      return messageElement;
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public static String toString( Object obj )
   throws SerializationException
   {
      return toString( obj, null, false );
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj  DOCUMENT_ME
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public static String toString( Object obj,
                                  QName  name )
   throws SerializationException
   {
      return toString( obj, name, false );
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj      DOCUMENT_ME
    * @param name     DOCUMENT_ME
    * @param nillable DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public static String toString( Object  obj,
                                  QName   name,
                                  boolean nillable )
   throws SerializationException
   {
      MessageElement messageElement = (MessageElement) toSOAPElement( obj, name, nillable );
      try
      {
         return AnyHelper.toString( messageElement );
      }
      catch ( Exception e )
      {
         throw new SerializationException( "genericSerializationError", e );
      }
   }
}