/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.handler.axis;

import org.apache.axis.AxisEngine;
import org.apache.axis.MessageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Host lookup used to support multi-homed host configurations
 */
public class ServiceHost
{
   /**
    * DOCUMENT_ME
    */
   public static final String PROXY_PORT_PROPERTY = "org.globus.wsrf.proxy.port";

   /**
    * DOCUMENT_ME
    */
   private static final int DEFAULT_PORT = 8080;

   /**
    * DOCUMENT_ME
    */
   private static final String DEFAULT_PROTOCOL = "http";
   private static String       hostCache = "";

   /**
    * DOCUMENT_ME
    */
   static Log LOG = LogFactory.getLog( ServiceHost.class.getName(  ) );

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws IOException DOCUMENT_ME
    */
   public static URL getBaseURL(  )
   throws IOException
   {
      return getBaseURL( MessageContext.getCurrentContext(  ) );
   }

   /**
    * DOCUMENT_ME
    *
    * @param context DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws IOException DOCUMENT_ME
    */
   public static URL getBaseURL( MessageContext context )
   throws IOException
   {
      URL             baseURL = null;
      ContainerConfig config = null;
      if ( context == null )
      {
         config     = ContainerConfig.getConfig(  );
         baseURL =
            new URL( getDefaultProtocol(  ),
                     getDefaultHost(  ),
                     getDefaultPort(  ),
                     "/" + config.getServicesBaseUrlPath(  ) );
      }
      else
      {
         config     = ContainerConfig.getConfig( context.getAxisEngine(  ) );
         URL    address = getEndpoint( context );
         int    port      = address.getPort(  );
         String proxyPort = System.getProperty( PROXY_PORT_PROPERTY );
         if ( proxyPort != null )
         {
            port = Integer.parseInt( proxyPort );
         }

         baseURL =
            new URL( address.getProtocol(  ),
                     ServiceHost.getHost( context ),
                     port,
                     "/" + config.getServicesBaseUrlPath(  ) );
      }

      return baseURL;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws IOException DOCUMENT_ME
    */
   public static URL getDefaultBaseURL(  )
   throws IOException
   {
      return getBaseURL( null );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws IOException DOCUMENT_ME
    */
   public static String getDefaultHost(  )
   throws IOException
   {
      synchronized ( ServiceHost.hostCache )
      {
         if ( ServiceHost.hostCache.equals( "" ) )
         {
            ServiceHost.hostCache = getHost( (AxisEngine) null );
         }

         return ServiceHost.hostCache;
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static int getDefaultPort(  )
   {
      return getPort( null );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static String getDefaultProtocol(  )
   {
      return getProtocol( null );
   }

   /**
    * DOCUMENT_ME
    *
    * @param context DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws IOException DOCUMENT_ME
    */
   public static String getHost( MessageContext context )
   throws IOException
   {
      return getHost( ( context == null ) ? null : context.getAxisEngine(  ) );
   }

   /**
    * DOCUMENT_ME
    *
    * @param engine DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws IOException DOCUMENT_ME
    */
   public static String getHost( AxisEngine engine )
   throws IOException
   {
      ContainerConfig config = null;
      if ( engine == null )
      {
         config = ContainerConfig.getConfig(  );
      }
      else
      {
         config = ContainerConfig.getConfig( engine );
      }

      InetAddress localHost = null;

      String      disableDNSConfig = config.getOption( ContainerConfig.DISABLE_DNS );

      boolean     disableDNS = ( disableDNSConfig == null ) ? false : disableDNSConfig.equalsIgnoreCase( "true" );

      String      logicalHostName = config.getOption( ContainerConfig.LOGICAL_HOST );
      if ( logicalHostName == null )
      {
         //logicalHostName = CoGProperties.getDefault().getHostName();
         //todo uncomment when we know what this is
      }

      String host = null;

      if ( !disableDNS )
      {
         // try to get ip for ogsa host
         try
         {
            String      ogsaHostName = config.getOption( ContainerConfig.WSRF_HOST );
            InetAddress ogsaHost = null;

            if ( ogsaHostName != null )
            {
               ogsaHost = InetAddress.getByName( ogsaHostName );
            }

            if ( ogsaHost != null )
            {
               String ogsaHostIP = ogsaHost.getHostAddress(  );

               if ( ogsaHostIP != null )
               {
                  return ogsaHostIP;
               }
            }
         }
         catch ( Exception e )
         {
            LOG.debug( "", e );
         }

         if ( logicalHostName != null )
         {
            localHost = InetAddress.getByName( logicalHostName );
         }
         else
         {
            localHost = InetAddress.getLocalHost(  );
         }

         // Determine whether we should publish IP or hostname
         String  publishHost = config.getOption( ContainerConfig.PUBLISH_HOST_NAME );

         boolean publishHostName = ( ( publishHost != null ) && publishHost.equalsIgnoreCase( "true" ) );

         if ( publishHostName )
         {
            host = localHost.getHostName(  );
         }
         else
         {
            host = localHost.getHostAddress(  );
         }
      }
      else if ( logicalHostName != null )
      {
         host = logicalHostName;
      }

      if ( host == null )
      {
         host = "localhost";
      }
      else if ( host.indexOf( "." ) == -1 )
      {
         String domainName = config.getOption( ContainerConfig.DOMAIN_NAME );

         if ( domainName != null )
         {
            host = host + "." + domainName;
         }
      }

      return host;
   }

   /**
    * DOCUMENT_ME
    *
    * @param context DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static int getPort( MessageContext context )
   {
      if ( context == null )
      {
         return DEFAULT_PORT;
      }

      URL url = null;
      try
      {
         url = getEndpoint( context );
      }
      catch ( IOException e )
      {
         LOG.debug( "", e );
         return DEFAULT_PORT;
      }

      int port = url.getPort(  );
      if ( port == -1 )
      {
         String protocol = url.getProtocol(  );
         if ( protocol.equalsIgnoreCase( "http" ) )
         {
            port = 80;
         }
         else if ( protocol.equalsIgnoreCase( "https" ) )
         {
            port = 443;
         }
         else
         {
            // unknown protocol
            port = DEFAULT_PORT;
         }
      }

      return port;
   }

   /**
    * DOCUMENT_ME
    *
    * @param context DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static String getProtocol( MessageContext context )
   {
      if ( context == null )
      {
         return DEFAULT_PROTOCOL;
      }

      URL url = null;
      try
      {
         url = getEndpoint( context );
      }
      catch ( IOException e )
      {
         LOG.debug( "", e );
         return DEFAULT_PROTOCOL;
      }

      return url.getProtocol(  );
   }

   private static URL getEndpoint( MessageContext messageContext )
   throws MalformedURLException
   {
      String address = (String) messageContext.getProperty( MessageContext.TRANS_URL );
      return new URL( address );
   }
}