/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.impl;

import org.apache.commons.lang.SerializationException;
import org.apache.ws.resource.InvalidResourceKeyException;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPHeaderElement;

/**
 * LOG-DONE
 * A {@link org.apache.ws.resource.ResourceKey} whose value is a {@link String}. It is required that the key SOAP header
 * be an XML type.
 *
 * @author Ian P. Springer
 */
public class SimpleTypeResourceKey
   extends AbstractResourceKey
{
   private SOAPHeaderElement m_headerElem;

    public SimpleTypeResourceKey( QName name, Object value )
    {
        super( name, value );
    }
    
   /**
    * Creates a new XmlBeanResourceKey.
    *
    * @param headerElem the SOAPElement containing the key. The element's name and namespace are used as key name.
    *
    * @throws org.apache.ws.resource.InvalidResourceKeyException
    *          if deserialization of the key fails.
    */
   public SimpleTypeResourceKey( SOAPHeaderElement headerElem )
   throws InvalidResourceKeyException
   {
      super( headerElem );
      m_headerElem = headerElem;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws org.apache.commons.lang.SerializationException
    *          DOCUMENT_ME
    */
   public SOAPElement toSOAPElement(  )
   throws SerializationException
   {
      return m_headerElem;
   }

    public Class getType()
    {
        return java.lang.String.class;
    }

    /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected final String getStringValue(  )
   {
      return (String) getValue(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param headerElem DOCUMENT_ME
    * @param type       DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws org.apache.commons.lang.SerializationException
    *          DOCUMENT_ME
    * @throws org.apache.ws.resource.InvalidResourceKeyException
    *          DOCUMENT_ME
    */
   protected Object deserializeKey( SOAPHeaderElement headerElem,
                                    Class             type )
   throws SerializationException, 
          InvalidResourceKeyException
   {
      try
      {
         XmlObject headerXBean = XmlBeanUtils.toXmlObject( headerElem );
         return XmlBeanUtils.getValue( headerXBean );
      }
      catch ( Exception e )
      {
         throw new SerializationException( e );
      }
   }
}