/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.impl;

import org.apache.commons.lang.SerializationException;
import org.apache.ws.resource.InvalidResourceKeyException;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;

import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPHeaderElement;

/**
 * LOG-DONE
 * A {@link org.apache.ws.resource.ResourceKey} whose value is an {@link XmlObject}.
 *
 * @author Ian P. Springer, Sal Campana
 */
public class XmlBeanResourceKey
   extends AbstractResourceKey
{
   /**
    * Creates a new XmlBeanResourceKey.
    *
    * @param headerElem the SOAPElement containing the key. The element's name and namespace are used as key name.
    *
    * @throws InvalidResourceKeyException if deserialization of the key fails.
    */
   public XmlBeanResourceKey( SOAPHeaderElement headerElem )
   throws InvalidResourceKeyException
   {
      super( headerElem );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public SOAPElement toSOAPElement(  )
   throws SerializationException
   {
      try
      {
         return XmlBeanUtils.toSOAPElement( (XmlObject) getValue(  ) );
      }
      catch ( Exception e )
      {
         throw new SerializationException( e );
      }
   }

    public Class getType()
    {
        return XmlObject.class;
    }

    /**
    * DOCUMENT_ME
    *
    * @param headerElem DOCUMENT_ME
    * @param type       DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    * @throws org.apache.ws.resource.InvalidResourceKeyException
    *                                DOCUMENT_ME
    */
   protected Object deserializeKey( SOAPHeaderElement headerElem,
                                    Class             type )
   throws SerializationException, 
          org.apache.ws.resource.InvalidResourceKeyException
   {
      try
      {
         return XmlBeanUtils.toXmlObject( headerElem );
      }
      catch ( Exception e )
      {
         throw new SerializationException( e );
      }
   }
}