/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.impl;

import org.apache.commons.lang.SerializationException;
import org.apache.ws.resource.InvalidResourceKeyException;

import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPHeaderElement;

/**
 * LOG-DONE
 * A {@link org.apache.ws.resource.ResourceKey} whose value is a {@link String} containing XML.
 *
 * @author Ian P. Springer, Sal Campana
 */
public class XmlStringResourceKey
   extends AbstractResourceKey
{
   private SOAPHeaderElement m_headerElem;

   /**
    * Creates a new XmlStringResourceKey.
    *
    * @param headerElem the SOAPElement containing the key. The element's name and namespace are used as key name.
    *
    * @throws InvalidResourceKeyException if deserialization of the key fails.
    */
   public XmlStringResourceKey( SOAPHeaderElement headerElem )
   throws InvalidResourceKeyException
   {
      super( headerElem );
      m_headerElem = headerElem;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException DOCUMENT_ME
    */
   public SOAPElement toSOAPElement(  )
   throws SerializationException
   {
      return m_headerElem;
   }

    public Class getType()
    {
        return String.class;
    }

    /**
    * DOCUMENT_ME
    *
    * @param headerElem DOCUMENT_ME
    * @param type       DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SerializationException      DOCUMENT_ME
    * @throws InvalidResourceKeyException DOCUMENT_ME
    */
   protected Object deserializeKey( SOAPHeaderElement headerElem,
                                    Class             type )
   throws SerializationException, 
          InvalidResourceKeyException
   {
      return headerElem.toString(  );
   }
}