/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.lifetime.faults;

import org.apache.ws.resource.ResourceUnknownException;
import org.apache.ws.resource.faults.AbstractBaseFaultException;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.ws.util.i18n.Messages;

import javax.xml.namespace.QName;

/**
 * LOG-DONE Indicates that a WS-Resource could not be destroyed for some reason.
 *
 * @author Ian P. Springer
 */
public class ResourceNotDestroyedFaultException
        extends AbstractBaseFaultException
{
    private static final Messages MSG = MessagesImpl.getInstance();

    private QName m_name;
    private Object m_resourceId;
    private String m_serviceName;

    /**
     * Constructs a new ResourceUnknownFaultException, specifying the resource id and service name of the unknown
     * resource.
     *
     * @param resourceId
     * @param serviceName
     */
    public ResourceNotDestroyedFaultException( NamespaceVersionHolder namespaces, Object resourceId,
                                               String serviceName )
    {
        super( namespaces, MSG.getMessage( Keys.UNABLE_TO_DESTROY_RESOURCE, serviceName, resourceId ) );
        m_name = new QName( namespaces.getLifetimeXsdNamespace(), "ResourceUnknownFault",
                "wsrl" );
        m_resourceId = resourceId;
        m_serviceName = serviceName;
    }

    /**
     * @param rue
     */
    public ResourceNotDestroyedFaultException( NamespaceVersionHolder namespaces, ResourceUnknownException rue )
    {
        // TODO
        this( namespaces, rue.getResourceId(),
                rue.getServiceName() );
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public Object getResourceId()
    {
        return m_resourceId;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public String getServiceName()
    {
        return m_serviceName;
    }

    /**
     * Returns the element name for this base fault.
     *
     * @return the element name for this base fault
     */
    public QName getBaseFaultName()
    {
        return m_name;
    }

}