/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.lifetime.v1_2_draft01.porttype.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.ResourceException;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.lifetime.ScheduledResourceTerminationResource;
import org.apache.ws.resource.lifetime.faults.UnableToSetTerminationTimeFaultException;
import org.apache.ws.resource.lifetime.v1_2_draft01.porttype.ScheduledResourceTerminationPortType;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.ws.resource.properties.impl.AbstractResourcePropertiesPortType;
import org.apache.ws.resource.properties.v1_2_draft01.impl.NamespaceVersionHolderImpl;
import org.apache.ws.resource.properties.v1_2_draft01.impl.NamespaceVersionHolderImpl;
import org.apache.ws.util.i18n.Messages;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceLifetime12Draft01.SetTerminationTimeDocument;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceLifetime12Draft01.SetTerminationTimeResponseDocument;

import java.util.Calendar;

/**
 * LOG-DONE
 * An implementation of the wsrlw:ScheduledResourceTermination portType.
 *
 * @author Ian P. Springer (Hewlett-Packard Company)
 */
public class ScheduledResourceTerminationPortTypeImpl
   extends AbstractResourcePropertiesPortType
   implements ScheduledResourceTerminationPortType
{
   /**
    * DOCUMENT_ME
    */
   private static final Log LOG = LogFactory.getLog( ScheduledResourceTerminationPortTypeImpl.class.getName(  ) );

   private static final Messages MSG = MessagesImpl.getInstance();

   private static final NamespaceVersionHolder NAMESPACE_SET = new NamespaceVersionHolderImpl() ;


   /**
    * Creates a new {@link ScheduledResourceTerminationPortTypeImpl} object.
    *
    * @param resourceContext DOCUMENT_ME
    */
   public ScheduledResourceTerminationPortTypeImpl( ResourceContext resourceContext )
   {
      super( resourceContext );
   }

    protected NamespaceVersionHolder getNamespaceSet()
    {
        return NAMESPACE_SET;
    }

    /**
    * DOCUMENT_ME
    *
    * @param requestDoc DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public SetTerminationTimeResponseDocument setTerminationTime( SetTerminationTimeDocument requestDoc )
   {
      SetTerminationTimeResponseDocument responseDoc       = createResponseDocument(  );
      Calendar                           currentTime       = Calendar.getInstance(  );
      Calendar                           requestedTermTime =
         requestDoc.getSetTerminationTime(  ).getRequestedTerminationTime(  );
      Calendar                           newTermTime = null;

      // if requested termination time is in the past, destroy immediately
      if ( ( requestedTermTime != null ) && requestedTermTime.getTime(  ).before( currentTime.getTime(  ) ) )
      {
         destroyResource(  );
         newTermTime = currentTime;
      }
      else
      {
         newTermTime = requestedTermTime;
      }

      getResourceState(  ).setTerminationTime( newTermTime );
      populateResponseDocument( responseDoc, currentTime, newTermTime );
      return responseDoc;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected final ScheduledResourceTerminationResource getResourceState(  )
   {
      return (ScheduledResourceTerminationResource) getResource(  );
   }

   private SetTerminationTimeResponseDocument createResponseDocument(  )
   {
      SetTerminationTimeResponseDocument responseDoc = SetTerminationTimeResponseDocument.Factory.newInstance(  );
      responseDoc.addNewSetTerminationTimeResponse(  );
      return responseDoc;
   }

   private void destroyResource(  )
   {
      LOG.debug( MSG.getMessage( Keys.DESTROYING_RESOURCE,getResourceContext(  ).getServiceName(  ),
                 String.valueOf( getResourceKey(  ))) );
      try
      {
         getResourceHome(  ).remove( getResourceKey(  ) );
      }
      catch ( ResourceException re )
      {
         throw new UnableToSetTerminationTimeFaultException( NAMESPACE_SET, null );
      }
   }

   private void populateResponseDocument( SetTerminationTimeResponseDocument responseDoc,
                                          Calendar                           currentTime,
                                          Calendar                           newTermTime )
   {
      SetTerminationTimeResponseDocument.SetTerminationTimeResponse response =
         responseDoc.getSetTerminationTimeResponse(  );
      response.setCurrentTime( currentTime );
      response.setNewTerminationTime( newTermTime );
   }
}