/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties;

import javax.xml.namespace.QName;

/**
 * Defines metadata information about a resource property.
 */
public interface ResourcePropertyMetaData
{
   /**
    * Returns the maximum number of values that this resource property can have.
    *
    * @return the maximum number of values that this resource property can have. Returns {@link Integer#MAX_VALUE
    *         Integer.MAX_VALUE} if unlimited.
    */
   int getMaxOccurs(  );

   /**
    * Returns the minimum number of values that this resource property can have.
    *
    * @return the minimum number of values that this resource property can have.
    */
   int getMinOccurs(  );

   /**
    * Returns the resource property name.
    *
    * @return the resource property name.
    */
   QName getName(  );

   /**
    * Returns if this resource property can be nillable.
    *
    * @return true if the resource property can be nillable. False otherwise.
    */
   boolean isNillable(  );

   /**
    * Returns whether this property is read only.
    *
    * @return Returns true if this property is read only. False, otherwise.
    */
   boolean isReadOnly(  );

   /**
    * Returns the Java element type of this property. When adding or setting element values the input value will be
    * automatically converted into this type. If set to <code>Object.class</code> then no conversion will be done.
    *
    * @return Element type of this property.
    */
   Class getType(  );

   /**
    * Creates a resource property of the type defined by this metadata, with the specified property set as its
    * container.
    */
   ResourceProperty create( ResourcePropertySet propSet );

   /**
    * Determines if the ResourceProperty is an XSD:Any    
    */
   boolean isAny();
}