/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties;

import org.apache.commons.lang.SerializationException;
import org.w3c.dom.Element;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import java.util.Iterator;

/**
 * This interface defines the API used to access a collection of resource properties contained in a resource.
 */
public interface ResourcePropertySet
{
   /**
    * Returns <tt>true</tt> if, and only if, this set contains no properties.
    *
    * @return <tt>true</tt> if, and only if, this set contains no properties
    */
   boolean isEmpty(  );

   /**
    * TODO
    *
    * @return
    */
   ResourcePropertySetMetaData getMetaData(  );

   /**
    * Adds a resource property to the set and makes it available to queries and subscriptions, etc.
    *
    * @param property the <code>ResourceProperty</code> to add.
    */
   boolean add( ResourceProperty property )
   throws MetaDataViolationException;

   /**
    * Removes all properties.
    */
   void clear(  )
   throws MetaDataViolationException;

   /**
    * Retrieves a resource property.
    *
    * @param propName the name of the <code>ResourceProperty</code> to retrieve.
    *
    * @return the <code>ResourceProperty</code> that was retrieved. Null if the property does not exist.
    */
   ResourceProperty get( QName propName );

   /**
    * Returns an iterator over <code>ResourceProperty</code> entries.
    *
    * @return an iterator over <code>ResourceProperty</code> entries.
    */
   Iterator iterator(  );

   /**
    * Removes a resource property.
    *
    * @param propName the name of the <code>ResourceProperty</code> to remove.
    *
    * @return the <code>ResourceProperty</code> that was removed. Null if the property does not exist or was not
    *         removed.
    */
   boolean remove( QName propName )
   throws MetaDataViolationException;

   /**
    * Returns the number of properties in this set.
    *
    * @return the number of properties
    */
   int size(  );

   /**
    * Returns the entire Resource Property document as a DOM Element.
    */
   Element toElement(  )
   throws SerializationException;

   /**
    * Returns the entire Resource Property document as a SOAPElement.
    */
   SOAPElement toSOAPElement(  )
   throws SerializationException;

   /**
    * Returns the XML representation of this property set (i.e. the resource properties document).
    *
    * @return the XML representation of this property set
    */
   String toXML(  );
}