/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.faults;

import org.apache.ws.Soap1_1Constants;
import org.apache.ws.resource.faults.AbstractBaseFaultException;
import org.apache.ws.resource.properties.NamespaceVersionHolder;

import javax.xml.namespace.QName;

/**
 * LOG-DONE WS-ResourceProperties SetResourcePropertyRequestFailed fault type implemented as a Java exception.
 *
 * @author Ian P. Springer
 */
public class SetResourcePropertyRequestFailedFaultException
        extends AbstractBaseFaultException
{

    private QName m_name;

    /**
     * Constructs a new SetResourcePropertyRequestFailedFaultException.
     *
     * @param namespaces
     * @param faultString a description of the error that occurred
     */
    public SetResourcePropertyRequestFailedFaultException( NamespaceVersionHolder namespaces, String faultString )
    {
        super( namespaces, faultString, Soap1_1Constants.FAULT_SERVER );
        m_name = new QName( namespaces.getPropertiesXsdNamespace(), "SetResourcePropertyRequestFailedFault",
                "wsrp" );
    }

    /**
     * Returns the element name for this base fault.
     *
     * @return the element name for this base fault
     */
    public QName getBaseFaultName()
    {
        return m_name;
    }
}