/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.faults;

import org.apache.ws.resource.faults.AbstractBaseFaultException;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.ws.util.i18n.Messages;

import javax.xml.namespace.QName;
import java.net.URI;

/**
 * LOG-DONE WS-ResourceProperties UnknownQueryExpressionDialect fault type implemented as a Java exception.
 *
 * @author Ian P. Springer
 */
public class UnknownQueryExpressionDialectFaultException
        extends AbstractBaseFaultException
{

    private static final Messages MSG = MessagesImpl.getInstance();

    private QName m_name;
    private URI m_dialect;

    /**
     * Constructs a new BaseFaultException with the specified SOAP message and actor.
     *
     * @param dialect the URI of the unsupported dialect
     */
    public UnknownQueryExpressionDialectFaultException( NamespaceVersionHolder namespaces, URI dialect )
    {
        super( namespaces, MSG.getMessage( Keys.DIALECT, dialect ) );
        m_name = new QName( namespaces.getPropertiesXsdNamespace(), "UnknownQueryExpressionDialectFault",
                "wsrp" );
        m_dialect = dialect;
    }

    /**
     * Returns the URI of the unsupported dialect.
     *
     * @return the URI of the unsupported dialect
     */
    public URI getDialect()
    {
        return m_dialect;
    }

    /**
     * Returns the element name for this base fault.
     *
     * @return the element name for this base fault
     */
    public QName getBaseFaultName()
    {
        return m_name;
    }

}