/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.impl;

import org.apache.ws.resource.properties.ResourceProperty;
import org.apache.ws.resource.properties.ResourcePropertyMetaData;
import org.apache.ws.resource.properties.ResourcePropertySet;
import org.apache.ws.resource.properties.ResourcePropertySetMetaData;
import org.apache.xmlbeans.SchemaProperty;
import org.oasisOpen.docs.wsrf.x2004.x10.wsrfWSResourceMetadataDescriptor10Draft01.ModifiabilityType;
import org.oasisOpen.docs.wsrf.x2004.x10.wsrfWSResourceMetadataDescriptor10Draft01.PropertyType;

import javax.xml.namespace.QName;
import java.math.BigInteger;

/**
 * Apache XMLBeans-based implementation of resource property metadata.
 *
 * @author Ian P. Springer
 */
public class XmlBeansResourcePropertyMetaData
        implements ResourcePropertyMetaData
{
    //private static final Log LOG = LogFactory.getLog( XmlBeansResourcePropertyMetaData.class );
    private SchemaProperty m_schemaProp;
    private boolean m_isReadOnly;
    private ResourcePropertySetMetaData m_setMetaData;

    /**
     * Creates a new {@link XmlBeansResourcePropertyMetaData} object.
     *
     * @param schemaProp DOCUMENT_ME
     */
    public XmlBeansResourcePropertyMetaData( SchemaProperty schemaProp )
    {
        this( schemaProp, false );
    }

    /**
     * Creates a new {@link XmlBeansResourcePropertyMetaData} object based on the specified
     * xsd:element schema property and wsrmd:Property.
     *
     * @param schemaProp DOCUMENT_ME
     * @param propDesc
     */
    public XmlBeansResourcePropertyMetaData( SchemaProperty schemaProp, PropertyType propDesc )
    {
        this( schemaProp, propDesc != null ? propDesc.getModifiability().intValue() == ModifiabilityType.INT_READ_ONLY : false );
    }

    /**
     * Creates a new {@link XmlBeansResourcePropertyMetaData} object.
     *
     * @param schemaProp DOCUMENT_ME
     * @param isReadOnly DOCUMENT_ME
     */
    public XmlBeansResourcePropertyMetaData( SchemaProperty schemaProp,
                                             boolean isReadOnly )
    {
        m_schemaProp = schemaProp;
        m_isReadOnly = isReadOnly;
    }
    
    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public int getMaxOccurs()
    {
        BigInteger maxOccurs = m_schemaProp.getMaxOccurs();
        return ( maxOccurs != null ) ? maxOccurs.intValue() : ( -1 );
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public int getMinOccurs()
    {
        return m_schemaProp.getMinOccurs().intValue();
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public QName getName()
    {
        return m_schemaProp.getName();
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public boolean isNillable()
    {
        return m_schemaProp.hasNillable() == SchemaProperty.CONSISTENTLY;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public boolean isReadOnly()
    {
        return m_isReadOnly;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public ResourcePropertySetMetaData getSetMetaData()
    {
        synchronized ( this )
        {
            if ( m_setMetaData == null )
            {
                m_setMetaData = new XmlBeansResourcePropertySetMetaData( m_schemaProp.getContainerType() );
            }
        }

        return m_setMetaData;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public Class getType()
    {
        return m_schemaProp.getType().getJavaClass();
    }

    /**
     * DOCUMENT_ME
     *
     * @param propSet DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public ResourceProperty create( ResourcePropertySet propSet )
    {
        return new XmlBeansResourceProperty( this, (XmlBeansResourcePropertySet) propSet );
    }

    public boolean isAny()
    {
        return false;
    }
}