/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.query.impl;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.properties.query.QueryExpression;
import org.apache.ws.util.JaxpUtils;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.ws.util.i18n.Messages;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceProperties12Draft01.QueryExpressionType;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import javax.xml.rpc.JAXRPCException;
import java.net.URI;

/**
 * LOG-DONE
 *
 * @author Ian P. Springer
 */
public class XmlBeansQueryExpression
        implements QueryExpression, XmlObjectWrapper
{
    private static final Log LOG = LogFactory.getLog( XmlBeansQueryExpression.class );
    public static final Messages MSG = MessagesImpl.getInstance();

    private XmlObject m_queryExprXBean;
    private URI m_dialect;
    private Node m_content;

    /**
     * Creates a new {@link XmlBeansQueryExpression} object.
     *
     * @param queryExprXBean DOCUMENT_ME
     *
     * @throws JAXRPCException          DOCUMENT_ME
     */
    public XmlBeansQueryExpression( QueryExpressionType queryExprXBean )
    {
        m_queryExprXBean = queryExprXBean;
        try
        {
            m_dialect = new URI( queryExprXBean.getDialect() );
        }
        catch ( Exception e )
        {
            throw new JAXRPCException( MSG.getMessage( Keys.FAILED_INIT_DIALECT, e ) );
        }

        XmlObject[] childElems = XmlBeanUtils.getChildElements( queryExprXBean );
        if ( childElems.length > 1 )
        {
            throw new JAXRPCException( MSG.getMessage( Keys.QUERY_ONLY_ONE_NODE ) );
        }

        if ( childElems.length == 1 )
        {
            m_content = ( (DocumentFragment) childElems[0].newDomNode() ).getFirstChild();
        }
        else
        {
            Node queryExprNode = ( (DocumentFragment) queryExprXBean.newDomNode( new XmlOptions().setSaveOuter() ) ).getFirstChild();
            m_content = queryExprNode.getFirstChild();
            if ( ( m_content == null ) || !( m_content instanceof Text ) )
            {
                throw new JAXRPCException( MSG.getMessage( Keys.QUERY_MUST_HAVE_ELEM_OR_TXT ) );
            }
        }
        if ( LOG.isDebugEnabled() )
        {
            LOG.debug( MSG.getMessage( Keys.QUERY_EXPR, toString() ) );
        }
    }

    public XmlBeansQueryExpression(org.oasisOpen.docs.wsrf.x2004.x11.wsrfWSResourceProperties12Draft05.QueryExpressionType queryExprElem)
    {
        m_queryExprXBean = queryExprElem;
        try
        {
            m_dialect = new URI( queryExprElem.getDialect() );
        }
        catch ( Exception e )
        {
            throw new JAXRPCException( MSG.getMessage( Keys.FAILED_INIT_DIALECT, e ) );
        }

        XmlObject[] childElems = XmlBeanUtils.getChildElements( queryExprElem );
        if ( childElems.length > 1 )
        {
            throw new JAXRPCException( MSG.getMessage( Keys.QUERY_ONLY_ONE_NODE ) );
        }

        if ( childElems.length == 1 )
        {
            m_content = ( (DocumentFragment) childElems[0].newDomNode() ).getFirstChild();
        }
        else
        {
            Node queryExprNode = ( (DocumentFragment) queryExprElem.newDomNode( new XmlOptions().setSaveOuter() ) ).getFirstChild();
            m_content = queryExprNode.getFirstChild();
            if ( ( m_content == null ) || !( m_content instanceof Text ) )
            {
                throw new JAXRPCException( MSG.getMessage( Keys.QUERY_MUST_HAVE_ELEM_OR_TXT ) );
            }
        }
        if ( LOG.isDebugEnabled() )
        {
            LOG.debug( MSG.getMessage( Keys.QUERY_EXPR, toString() ) );
        }
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public Node getContent()
    {
        return m_content;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public URI getDialect()
    {
        return m_dialect;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public String toString()
    {
        String contentString;
        try
        {
            contentString = JaxpUtils.toString( m_content );
        }
        catch ( Exception e )
        {
            contentString = MSG.getMessage( Keys.NAME ) + "=" + m_content.getNodeName() + ", " +
                    MSG.getMessage( Keys.VALUE ) +
                    "=" +
                    m_content.getNodeValue();
        }

        return new ToStringBuilder( this ).append( MSG.getMessage( Keys.DIALECT ), m_dialect ).append(
                MSG.getMessage( Keys.CONTENT ), contentString )
                .toString();
    }

    public XmlObject getXmlObject()
    {
        return m_queryExprXBean;
    }
}