/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.query.xpath;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.properties.ResourcePropertySet;
import org.apache.ws.resource.properties.query.ExpressionEvaluator;
import org.apache.ws.resource.properties.query.InvalidQueryExpressionException;
import org.apache.ws.resource.properties.query.QueryEvaluationErrorException;
import org.apache.ws.resource.properties.query.QueryExpression;
import org.apache.ws.resource.properties.query.UnknownQueryExpressionDialectException;
import org.apache.ws.util.i18n.Messages;
import org.w3c.dom.Element;

/**
 * LOG-DONE
 * @author Ian P. Springer
 */
public abstract class AbstractXPathExpressionEvaluator
   implements ExpressionEvaluator
{

   private static final Log LOG = LogFactory.getLog( AbstractXPathExpressionEvaluator.class.getName(  ) );
   public static final Messages MSG = MessagesImpl.getInstance();
   /**
    * DOCUMENT_ME
    *
    * @param queryExpr           DOCUMENT_ME
    * @param resourcePropertySet DOCUMENT_ME
    * @param nsContext           DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws UnknownQueryExpressionDialectException
    *                                       DOCUMENT_ME
    * @throws QueryEvaluationErrorException DOCUMENT_ME
    * @throws InvalidQueryExpressionException
    *                                       DOCUMENT_ME
    */
   public Object evaluate( QueryExpression     queryExpr,
                           ResourcePropertySet resourcePropertySet,
                           Element             nsContext )
   throws UnknownQueryExpressionDialectException, 
          QueryEvaluationErrorException, 
          InvalidQueryExpressionException
   {
      XPathExpression xpathExpr = new XPathExpression( queryExpr, nsContext );
      if ( LOG.isDebugEnabled(  ) )
      {
         LOG.debug( MSG.getMessage( Keys.XPATH, xpathExpr.getValue(  ) ));
         LOG.debug( MSG.getMessage( Keys.PROP_DOC,resourcePropertySet.toXML() ));
      }
      return evaluate( xpathExpr, resourcePropertySet );
   }

   /**
    * DOCUMENT_ME
    *
    * @param expr            DOCUMENT_ME
    * @param resourcePropSet DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws UnknownQueryExpressionDialectException
    *                                       DOCUMENT_ME
    * @throws QueryEvaluationErrorException DOCUMENT_ME
    * @throws InvalidQueryExpressionException
    *                                       DOCUMENT_ME
    */
   protected abstract Object evaluate( XPathExpression     expr,
                                       ResourcePropertySet resourcePropSet )
   throws UnknownQueryExpressionDialectException, 
          QueryEvaluationErrorException, 
          InvalidQueryExpressionException;
}