/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.query.xpath;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.properties.faults.UnknownQueryExpressionDialectFaultException;
import org.apache.ws.resource.properties.query.InvalidQueryExpressionException;
import org.apache.ws.resource.properties.query.QueryConstants;
import org.apache.ws.resource.properties.query.QueryExpression;
import org.apache.ws.resource.properties.query.UnknownQueryExpressionDialectException;
import org.apache.ws.util.JaxpUtils;
import org.apache.ws.util.i18n.Messages;
import org.w3c.dom.Element;

/**
 * LOG-DONE
 * An XPath expression.
 *
 * @author Ian P. Springer
 */
public class XPathExpression
{
   private static final Log LOG = LogFactory.getLog( XPathExpression.class );
   public static final Messages MSG = MessagesImpl.getInstance();
   private String  m_value;
   private Element m_nsContext;

   /**
    * Creates a new {@link XPathExpression} object.
    *
    * @param queryExpr DOCUMENT_ME
    * @param nsContext DOCUMENT_ME
    *
    * @throws UnknownQueryExpressionDialectFaultException
    *          DOCUMENT_ME
    */
   public XPathExpression( QueryExpression queryExpr,
                           Element         nsContext )
   throws InvalidQueryExpressionException, 
          UnknownQueryExpressionDialectException
   {
      if ( !queryExpr.getDialect(  ).equals( QueryConstants.DIALECT_URI__XPATH1_0 ) )
      {
         throw new UnknownQueryExpressionDialectException( queryExpr.getDialect(  ) );
      }
      m_value = queryExpr.getContent(  ).getNodeValue(  );
      if ( m_value == null )
      {
         throw new InvalidQueryExpressionException( queryExpr );
      }
      m_value = m_value.trim();
      if ( m_value.length() == 0 )
      {
         throw new InvalidQueryExpressionException( queryExpr );
      }
      m_nsContext = nsContext;

      if(LOG.isDebugEnabled())
      {
        LOG.debug(MSG.getMessage( Keys.CONSTR_XPATH_EXPR,toString()));
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Element getNamespaceContext(  )
   {
      return m_nsContext;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getValue(  )
   {
      return m_value;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String toString(  )
   {
      String nsContextString;
      try
      {
         nsContextString = m_nsContext.toString();
      }
      catch ( Exception e )
      {
         nsContextString = MSG.getMessage( Keys.NAME)+"=" + m_nsContext.getNodeName(  ) + ", "+ MSG.getMessage( Keys.VALUE)+"=" + m_nsContext.getNodeValue(  );
      }

      return new ToStringBuilder( this ).append( MSG.getMessage( Keys.VALUE), m_value ).append( MSG.getMessage( Keys.NMSPCECTX), nsContextString )
                                        .toString(  );
   }
}