/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.v1_2_draft01.porttype.impl;

import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.ws.resource.properties.ResourceProperty;
import org.apache.ws.resource.properties.faults.InvalidResourcePropertyQNameFaultException;
import org.apache.ws.resource.properties.impl.AbstractGetResourcePropertiesPortType;
import org.apache.ws.resource.properties.v1_2_draft01.porttype.GetMultipleResourcePropertiesPortType;
import org.apache.ws.resource.properties.v1_2_draft01.impl.NamespaceVersionHolderImpl;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceProperties12Draft01.GetMultipleResourcePropertiesDocument;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceProperties12Draft01.GetMultipleResourcePropertiesResponseDocument;

import javax.xml.namespace.QName;

/**
 * LOG-DONE An operation provider for wsrp:GetMultipleResourceProperties.
 *
 * @author Ian P. Springer
 */
public class GetMultipleResourcePropertiesPortTypeImpl
        extends AbstractGetResourcePropertiesPortType
        implements GetMultipleResourcePropertiesPortType
{

    private static final NamespaceVersionHolder NAMESPACE_SET = new NamespaceVersionHolderImpl() ;

    /**
     * Creates a new {@link GetMultipleResourcePropertiesPortTypeImpl} object.
     *
     * @param resourceContext DOCUMENT_ME
     */
    public GetMultipleResourcePropertiesPortTypeImpl( ResourceContext resourceContext )
    {
        super( resourceContext );
    }

    /**
     * Implementation of the wsrp:GetMultipleResourceProperties operation.
     *
     * @param requestDoc the requestDoc XMLBean
     *
     * @return the response XMLBean
     *
     * @throws InvalidResourcePropertyQNameFaultException
     *
     * @throws org.apache.ws.resource.faults.ResoureKeyHeaderNotFoundFaultException
     */
    public GetMultipleResourcePropertiesResponseDocument getMultipleResourceProperties( GetMultipleResourcePropertiesDocument requestDoc )
    {
        GetMultipleResourcePropertiesResponseDocument responseDoc = createResponseDocument();
        QName[] propNames = requestDoc.getGetMultipleResourceProperties().getResourcePropertyArray();
        ResourceProperty[] props = getMultipleProperties( propNames );
        addPropertiesToResponse( props, responseDoc.getGetMultipleResourcePropertiesResponse() );
        return responseDoc;
    }

    private GetMultipleResourcePropertiesResponseDocument createResponseDocument()
    {
        GetMultipleResourcePropertiesResponseDocument responseDoc = GetMultipleResourcePropertiesResponseDocument.Factory.newInstance();
        responseDoc.addNewGetMultipleResourcePropertiesResponse();
        return responseDoc;
    }

    protected NamespaceVersionHolder getNamespaceSet()
    {
        return NAMESPACE_SET;
    }

}