/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.tool.velocity;

import org.apache.ws.resource.Resource;
import org.apache.ws.resource.ResourceDefinition;
import org.apache.ws.resource.handler.WsrfService;
import org.apache.ws.resource.tool.PortType2JavaInfo;

import javax.xml.namespace.QName;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * TODO
 *
 * @author Ian Springer (ian DOT springer AT hp DOT com)
 */
public class ImplementsListBuilder
{

    public static String getServiceImplementsList( ResourceDefinition resourceDef, Map ptInfoMap )
    {
        Iterator ptNameIter = resourceDef.getImplementedResourceCapabilities().keySet().iterator();
        List interfaceNames = new ArrayList();
        interfaceNames.add( WsrfService.class.getName() );
        while ( ptNameIter.hasNext() )
        {
            QName ptName = (QName) ptNameIter.next();
            PortType2JavaInfo ptInfo = (PortType2JavaInfo) ptInfoMap.get( ptName );
            if ( ptInfo != null && ptInfo.getServiceInterfaceName() != null )
            {
               interfaceNames.add( ptInfo.getServiceInterfaceName() );
            }
        }
        return toCommaSeparatedString( interfaceNames );
    }

    public static String getResourceImplementsList( ResourceDefinition resourceDef, Map ptInfoMap )
    {
        Iterator ptNameIter = resourceDef.getImplementedResourceCapabilities().keySet().iterator();
        List interfaceNames = new ArrayList();
        while ( ptNameIter.hasNext() )
        {
            QName ptName = (QName) ptNameIter.next();
            PortType2JavaInfo ptInfo = (PortType2JavaInfo) ptInfoMap.get( ptName );
            if ( ptInfo != null && ptInfo.getResourceInterfaceName() != null )
            {
               interfaceNames.add( ptInfo.getResourceInterfaceName() );
            }
        }
        if ( interfaceNames.isEmpty() )
        {
            interfaceNames.add( Resource.class.getName() );
        }
        return toCommaSeparatedString( interfaceNames );
    }

    private static String toCommaSeparatedString( List list )
    {
        StringBuffer buf = new StringBuffer();
        for ( int i = 0; i < list.size() - 1; i++ )
        {
            buf.append( list.get( i ) );
            buf.append( ", " );
        }
        buf.append( list.get( list.size() - 1 ) );
        return buf.toString();
    }

}
