/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util;

import javax.xml.namespace.QName;

/**
 * @author Ian P. Springer (Hewlett-Packard Company)
 */
public interface XmlConstants
{
   /* =========================  Namespace URIs and Prefixes  ========================== */

   /**
    * Prefix representing default XML Namespace.
    */
   String NSPREFIX_DEFAULT = "";

   /**
    * DOCUMENT_ME
    */
   String NSPREFIX_SCHEMA_XSD = "xsd";

   /**
    * DOCUMENT_ME
    */
   String NSPREFIX_SCHEMA_XSI = "xsi";

   /**
    * DOCUMENT_ME
    */
   String NSPREFIX_XML = "xml";

   /**
    * DOCUMENT_ME
    */
   String NSPREFIX_XMLNS = "xmlns";

   /**
    * XML-Schema NS-URI and Prefix.
    */
   String NSURI_SCHEMA_XSD = "http://www.w3.org/2001/XMLSchema";

   /**
    * XML-Schema-Instance NS-URI and Prefix.
    */
   String NSURI_SCHEMA_XSI = "http://www.w3.org/2001/XMLSchema-instance";

   /**
    * XML NS-URI and Prefix.
    */
   String NSURI_XML = "http://www.w3.org/XML/1998/namespace";

   /**
    * XML-Namespace NS-URI and Prefix.
    */
   String NSURI_XMLNS = "http://www.w3.org/2000/xmlns/";

   /**
    * standard XML prolog
    */
   String XML_PROLOG = "<?xml version=\"1.0\"?>";

   /* ============================  Type Names  ============================= */

   /**
    * DOCUMENT_ME
    */
   QName XSD_STRING = new QName( NSURI_SCHEMA_XSD, "string" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_BOOLEAN = new QName( NSURI_SCHEMA_XSD, "boolean" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_DOUBLE = new QName( NSURI_SCHEMA_XSD, "double" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_FLOAT = new QName( NSURI_SCHEMA_XSD, "float" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_INT = new QName( NSURI_SCHEMA_XSD, "int" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_INTEGER = new QName( NSURI_SCHEMA_XSD, "integer" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_LONG = new QName( NSURI_SCHEMA_XSD, "long" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_SHORT = new QName( NSURI_SCHEMA_XSD, "short" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_BYTE = new QName( NSURI_SCHEMA_XSD, "byte" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_DECIMAL = new QName( NSURI_SCHEMA_XSD, "decimal" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_BASE64 = new QName( NSURI_SCHEMA_XSD, "base64Binary" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_HEXBIN = new QName( NSURI_SCHEMA_XSD, "hexBinary" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_ANYSIMPLETYPE = new QName( NSURI_SCHEMA_XSD, "anySimpleType" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_ANYTYPE = new QName( NSURI_SCHEMA_XSD, "anyType" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_ANY = new QName( NSURI_SCHEMA_XSD, "any" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_QNAME = new QName( NSURI_SCHEMA_XSD, "QName" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_DATETIME = new QName( NSURI_SCHEMA_XSD, "dateTime" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_DATE = new QName( NSURI_SCHEMA_XSD, "date" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_TIME = new QName( NSURI_SCHEMA_XSD, "time" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_NORMALIZEDSTRING = new QName( NSURI_SCHEMA_XSD, "normalizedString" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_TOKEN = new QName( NSURI_SCHEMA_XSD, "token" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_UNSIGNEDLONG = new QName( NSURI_SCHEMA_XSD, "unsignedLong" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_UNSIGNEDINT = new QName( NSURI_SCHEMA_XSD, "unsignedInt" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_UNSIGNEDSHORT = new QName( NSURI_SCHEMA_XSD, "unsignedShort" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_UNSIGNEDBYTE = new QName( NSURI_SCHEMA_XSD, "unsignedByte" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_POSITIVEINTEGER = new QName( NSURI_SCHEMA_XSD, "positiveInteger" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_NEGATIVEINTEGER = new QName( NSURI_SCHEMA_XSD, "negativeInteger" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_NONNEGATIVEINTEGER = new QName( NSURI_SCHEMA_XSD, "nonNegativeInteger" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_NONPOSITIVEINTEGER = new QName( NSURI_SCHEMA_XSD, "nonPositiveInteger" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_YEARMONTH = new QName( NSURI_SCHEMA_XSD, "gYearMonth" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_MONTHDAY = new QName( NSURI_SCHEMA_XSD, "gMonthDay" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_YEAR = new QName( NSURI_SCHEMA_XSD, "gYear" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_MONTH = new QName( NSURI_SCHEMA_XSD, "gMonth" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_DAY = new QName( NSURI_SCHEMA_XSD, "gDay" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_DURATION = new QName( NSURI_SCHEMA_XSD, "duration" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_NAME = new QName( NSURI_SCHEMA_XSD, "Name" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_NCNAME = new QName( NSURI_SCHEMA_XSD, "NCName" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_NMTOKEN = new QName( NSURI_SCHEMA_XSD, "NMTOKEN" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_NMTOKENS = new QName( NSURI_SCHEMA_XSD, "NMTOKENS" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_NOTATION = new QName( NSURI_SCHEMA_XSD, "NOTATION" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_ENTITY = new QName( NSURI_SCHEMA_XSD, "ENTITY" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_ENTITIES = new QName( NSURI_SCHEMA_XSD, "ENTITIES" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_IDREF = new QName( NSURI_SCHEMA_XSD, "IDREF" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_IDREFS = new QName( NSURI_SCHEMA_XSD, "IDREFS" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_ANYURI = new QName( NSURI_SCHEMA_XSD, "anyURI" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_LANGUAGE = new QName( NSURI_SCHEMA_XSD, "language" );

   /**
    * DOCUMENT_ME
    */
   QName XSD_ID = new QName( NSURI_SCHEMA_XSD, "ID" );

   /* =========================  Elements and Attribute Names  ========================== */

   /**
    * DOCUMENT_ME
    */
   QName XSD_SCHEMA = new QName( NSURI_SCHEMA_XSD, "schema" );

   /**
    * DOCUMENT_ME
    */
   String XSD_ATTRIB_IMPORT = "import";

   /**
    * DOCUMENT_ME
    */
   String XSD_ATTRIB_INCLUDE = "include";

   /**
    * DOCUMENT_ME
    */
   String XSD_ATTRIB_SCHEMALOCATION = "schemaLocation";
}