/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util;

import org.apache.axis.utils.XMLUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.util.i18n.Keys;
import org.apache.ws.util.i18n.Messages;
import org.apache.ws.util.i18n.MessagesImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

/**
 * LOG-DONE
 * DOCUMENT_ME
 *
 * @author $author$
 * @version $Revision: 1.5 $
 */
public class XmlUtils
{
   private static Log                    LOG = LogFactory.getLog( XmlUtils.class.getName(  ) );
   public static final Messages MSG = MessagesImpl.getInstance();
   private static DocumentBuilderFactory s_dbf = getDOMFactory(  );

   /**
    * Get an empty new Document
    *
    * @return Document
    *
    * @throws ParserConfigurationException if construction problems occur
    */
   public static Document newDocument(  )
   throws ParserConfigurationException
   {
      synchronized ( s_dbf )
      {
         return s_dbf.newDocumentBuilder(  ).newDocument(  );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @param inp DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws ParserConfigurationException DOCUMENT_ME
    * @throws SAXException                 DOCUMENT_ME
    * @throws IOException                  DOCUMENT_ME
    */
   public static Document newDocument( InputSource inp )
   throws ParserConfigurationException, 
          SAXException, 
          IOException
   {
      DocumentBuilder db;

      synchronized ( s_dbf )
      {
         db = s_dbf.newDocumentBuilder(  );
      }

      db.setErrorHandler( new XMLUtils.ParserErrorHandler(  ) );
      return db.parse( inp );
   }

   /**
    * DOCUMENT_ME
    *
    * @param inp DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws ParserConfigurationException DOCUMENT_ME
    * @throws SAXException                 DOCUMENT_ME
    * @throws IOException                  DOCUMENT_ME
    */
   public static Document newDocument( InputStream inp )
   throws ParserConfigurationException, 
          SAXException, 
          IOException
   {
      return newDocument( new InputSource( inp ) );
   }

   /**
    * DOCUMENT_ME
    *
    * @param uri DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws ParserConfigurationException DOCUMENT_ME
    * @throws SAXException                 DOCUMENT_ME
    * @throws IOException                  DOCUMENT_ME
    */
   public static Document newDocument( String uri )
   throws ParserConfigurationException, 
          SAXException, 
          IOException
   {
      // call the authenticated version as there might be
      // username/password info embeded in the uri.
      return newDocument( uri, null, null );
   }

   /**
    * Create a new document from the given URI, use the username and password if the URI requires authentication.
    */
   public static Document newDocument( String uri,
                                       String username,
                                       String password )
   throws ParserConfigurationException, 
          SAXException, 
          IOException
   {
      InputSource ins = getInputSourceFromURI( uri, username, password );

      Document    doc = null;

      try
      {
         doc = newDocument( ins );
      }
      finally
      {
         if ( ins.getByteStream(  ) != null )
         {
            ins.getByteStream(  ).close(  );
         }
         else if ( ins.getCharacterStream(  ) != null )
         {
            ins.getCharacterStream(  ).close(  );
         }
      }

      return doc;
   }

   /**
    * DOCUMENT_ME
    *
    * @param doc DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static String toString( Document doc )
   {
      return XMLUtils.DocumentToString( doc );
   }

   /**
    * DOCUMENT_ME
    *
    * @param element DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static String toString( Element element )
   {
      return XMLUtils.ElementToString( element );
   }

   private static DocumentBuilderFactory getDOMFactory(  )
   {
      DocumentBuilderFactory dbf;

      try
      {
         dbf = DocumentBuilderFactory.newInstance(  );
         dbf.setNamespaceAware( true );
      }
      catch ( Exception e )
      {
         LOG.error( "", e );
         dbf = null;
      }

      return ( dbf );
   }

   /**
    * Utility to get the bytes at a protected uri
    * <p/>
    * This will retrieve the URL if a username and password are provided. The java.net.URL class does not do Basic
    * Authentication, so we have to do it manually in this routine.
    * <p/>
    * If no username is provided, we create an InputSource from the uri and let the InputSource go fetch the contents.
    *
    * @param uri      the resource to get
    * @param username basic auth username
    * @param password basic auth password
    */
   private static InputSource getInputSourceFromURI( String uri,
                                                     String username,
                                                     String password )
   throws IOException
   {
      URL wsdlurl = null;
      LOG.debug(MSG.getMessage( Keys.INPUT_SRC_FRM_URI,uri,username,password));
      try
      {
         wsdlurl = new URL( uri );
      }
      catch ( MalformedURLException e )
      {
         // we can't process it, it might be a 'simple' foo.wsdl
         // let InputSource deal with it
         return new InputSource( uri );
      }

      // if no authentication, just let InputSource deal with it
      if ( ( username == null ) && ( wsdlurl.getUserInfo(  ) == null ) )
      {
         return new InputSource( uri );
      }

      // if this is not an HTTP{S} url, let InputSource deal with it
      if ( !wsdlurl.getProtocol(  ).startsWith( "http" ) )
      {
         return new InputSource( uri );
      }

      URLConnection connection = wsdlurl.openConnection(  );

      // Does this work for https???
      if ( !( connection instanceof HttpURLConnection ) )
      {
         // can't do http with this URL, let InputSource deal with it
         return new InputSource( uri );
      }

      HttpURLConnection uconn    = (HttpURLConnection) connection;
      String            userinfo = wsdlurl.getUserInfo(  );
      uconn.setRequestMethod( "GET" );
      uconn.setAllowUserInteraction( false );
      uconn.setDefaultUseCaches( false );
      uconn.setDoInput( true );
      uconn.setDoOutput( false );
      uconn.setInstanceFollowRedirects( true );
      uconn.setUseCaches( false );

      // username/password info in the URL overrides passed in values
      String auth = null;

      if ( userinfo != null )
      {
         auth = userinfo;
      }
      else if ( username != null )
      {
         auth = ( password == null ) ? username : ( username + ":" + password );
      }

      if ( auth != null )
      {
         uconn.setRequestProperty( "Authorization",
                                   "Basic " + XMLUtils.base64encode( auth.getBytes( XMLUtils.getEncoding(  ) ) ) );
      }

      uconn.connect(  );

      return new InputSource( uconn.getInputStream(  ) );
   }
}