/*
 * This file or a portion of this file is licensed under the terms of the
 * Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without modifications,
 * you must include this notice in the file.
 */
package org.apache.ws.util.jndi;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NamingException;
import java.util.Hashtable;

/**
 * LOG-DONE
 */
public class BasicBeanFactory extends org.apache.naming.factory.BeanFactory {
    
   private static final Log LOG = LogFactory.getLog( BasicBeanFactory.class );
    /**
     * Create a new Bean instance. If the created bean implements
     * <code>Initializable</code> interface, the <code>initialize()</code>
     * function will be called after the bean is created and all properties
     * are set.
     */
    public Object getObjectInstance(Object obj, Name name, Context nameCtx,
                                    Hashtable environment)
        throws NamingException {
        LOG.debug( "Creating instance of " + obj.getClass().getName() + " bean " + " and adding to JNDI registry with name " + name + " ..." );
        Object bean = super.getObjectInstance(obj, name, nameCtx, environment);
        if (bean instanceof Initializable) {
            try {
                LOG.debug("Calling init() on the bean: " + bean.getClass().getName());
                ((Initializable)bean).init();
            } catch (Exception e) {
                NamingException ex =
                    new NamingException("beanInitFailed");
                ex.setRootCause(e);
                throw ex;
            }
        }
        return bean;
    }
    
}
