/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util.jndi.tools;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/** LOG-DONE
 * DOCUMENT_ME
 *
 * @author $author$
 * @version $Revision: 1.4 $
 */
public class ConfigContext
{
   /**
    * DOCUMENT_ME
    */
   protected Map subContexts = new HashMap(  );

   /**
    * DOCUMENT_ME
    */
   protected Map environmentEntries = new HashMap(  );

   /**
    * DOCUMENT_ME
    */
   protected Map resourceEntries = new HashMap(  );

   /**
    * DOCUMENT_ME
    */
   protected Map resourceLinks = new HashMap(  );

   /**
    * DOCUMENT_ME
    */
   protected String name = null;

   /**
    * DOCUMENT_ME
    */
   protected boolean global;

   /**
    * Creates a new {@link ConfigContext} object.
    */
   public ConfigContext(  )
   {
      this( false );
   }

   /**
    * Creates a new {@link ConfigContext} object.
    *
    * @param global DOCUMENT_ME
    */
   public ConfigContext( boolean global )
   {
      this.global = global;
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Environment getEnvironment( String name )
   {
      return (Environment) this.environmentEntries.get( name );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Set getEnvironmentNames(  )
   {
      return this.environmentEntries.keySet(  );
   }

   /**
    * @param global The global to set.
    */
   public void setGlobal( boolean global )
   {
      this.global = global;
   }

   /**
    * @return Returns the global.
    */
   public boolean isGlobal(  )
   {
      return this.global;
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    */
   public void setName( String name )
   {
      this.name = name;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getName(  )
   {
      return this.name;
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Resource getResource( String name )
   {
      return (Resource) this.resourceEntries.get( name );
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public ResourceLink getResourceLink( String name )
   {
      return (ResourceLink) this.resourceLinks.get( name );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Set getResourceLinkNames(  )
   {
      return this.resourceLinks.keySet(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Set getResourceNames(  )
   {
      return this.resourceEntries.keySet(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public ConfigContext getSubContext( String name )
   {
      return (ConfigContext) this.subContexts.get( name );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Set getSubContextNames(  )
   {
      return this.subContexts.keySet(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param environment DOCUMENT_ME
    */
   public void addEnvironment( Environment environment )
   {
      this.environmentEntries.put( environment.getName(  ),
                                   environment );
   }

   /**
    * DOCUMENT_ME
    *
    * @param resource DOCUMENT_ME
    */
   public void addResource( Resource resource )
   {
      this.resourceEntries.put( resource.getName(  ),
                                resource );
   }

   /**
    * DOCUMENT_ME
    *
    * @param link DOCUMENT_ME
    */
   public void addResourceLink( ResourceLink link )
   {
      this.resourceLinks.put( link.getName(  ),
                              link );
   }

   /**
    * DOCUMENT_ME
    *
    * @param context DOCUMENT_ME
    */
   public void addSubContext( ConfigContext context )
   {
      this.subContexts.put( context.getName(  ),
                            context );
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    */
   public void removeSubContext( String name )
   {
      this.subContexts.remove( name );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String toString(  )
   {
      StringBuffer out          = new StringBuffer(  );
      Set          names;
      Iterator     nameIterator;

      if ( global == true )
      {
         out.append( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n" );
         out.append( "<jndiConfig xmlns=\"http://wsrf.globus.org/jndi/config\">\r\n" );
         out.append( "    <global>\r\n" );
      }
      else
      {
         out.append( "    <service name=\"" );
         out.append( this.name );
         out.append( "\">\r\n" );
      }

      names           = this.getEnvironmentNames(  );
      nameIterator    = names.iterator(  );

      while ( nameIterator.hasNext(  ) )
      {
         out.append( ( (Environment) this.environmentEntries.get( nameIterator.next(  ) ) ).toString(  ) );
      }

      names           = this.getResourceNames(  );
      nameIterator    = names.iterator(  );

      while ( nameIterator.hasNext(  ) )
      {
         out.append( ( (Resource) this.resourceEntries.get( nameIterator.next(  ) ) ).toString(  ) );
      }

      names           = this.getResourceLinkNames(  );
      nameIterator    = names.iterator(  );

      while ( nameIterator.hasNext(  ) )
      {
         out.append( ( (ResourceLink) this.resourceLinks.get( nameIterator.next(  ) ) ).toString(  ) );
      }

      if ( global == true )
      {
         out.append( "    </global>\r\n" );

         //process sub contexts
         names           = this.getSubContextNames(  );
         nameIterator    = names.iterator(  );

         while ( nameIterator.hasNext(  ) )
         {
            out.append( ( (ConfigContext) this.subContexts.get( nameIterator.next(  ) ) ).toString(  ) );
         }

         out.append( "</jndiConfig>\r\n" );
      }
      else
      {
         out.append( "    </service>\r\n" );
      }

      return out.toString(  );
   }
}