/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util.soap;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.ws.util.i18n.Keys;
import org.apache.ws.util.i18n.Messages;
import org.apache.ws.util.i18n.MessagesImpl;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

/**
 * LOG-DONE
 * Ant task that wraps {@link SoapClient#sendRequest(java.net.URL, java.io.File, java.net.URI)}.
 *
 * @author Ian P. Springer (Hewlett-Packard Company)
 */
public class SoapClientTask extends Task
{
   public static final Messages MSG = MessagesImpl.getInstance();
   private URL m_serviceURL;
   private File m_requestFile;
   private URI m_soapAction;

   public void setServiceURL( URL serviceURL )
   {
      m_serviceURL = serviceURL;
   }

   public void setRequestFile( File requestFile )
   {
      m_requestFile = requestFile;
   }

   public void setSoapAction( String soapAction ) throws URISyntaxException
   {
      if ( ! isUnresolvedProperty( soapAction ) )
      {
         m_soapAction = new URI( soapAction );
      }
   }

   public void execute() throws BuildException
   {
      if ( m_serviceURL == null )
      {
         throw new BuildException( MSG.getMessage( Keys.SRVC_URL_REQ) );
      }
      if ( m_requestFile == null )
      {
         throw new BuildException( MSG.getMessage( Keys.RQST_FILE_REQ) );
      }
      String response;
      try
      {
         response = SoapClient.sendRequest( m_serviceURL, m_requestFile, m_soapAction );
      }
      catch ( IOException ioe )
      {
         throw new BuildException( ioe );
      }
      System.out.println( response );
   }

   boolean isUnresolvedProperty( String value )
   {
       return value.startsWith( "${" ) && value.endsWith( "}" );
   }

}
