/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util.test.axis;

import junit.framework.TestCase;

/**
 * A JUnit test case superclass that will setup each individual test with its own
 * Axis server.  Multiple Axis servers will be started/stopped - one per test method.
 *
 * @author mazz
 */
public abstract class AbstractMultipleAxisTestCase
   extends AbstractAxisTestCase
{
   /**
    * @see AbstractAxisTestCase#AbstractAxisTestCase()
    */
   public AbstractMultipleAxisTestCase(  )
   {
       super();
   }

   /**
    * @see AbstractAxisTestCase#AbstractAxisTestCase(boolean)
   */
   public AbstractMultipleAxisTestCase(boolean runServer )
   {
       super(runServer);       
   }

   /**
    * @see AbstractAxisTestCase#AbstractAxisTestCase(String)
    */
   public AbstractMultipleAxisTestCase( String name )
   {
      super( name );
   }

   /**
    * Starts the embedded Axis server.
    *
    * @see TestCase#setUp()
    */
   protected void setUp(  )
   throws Exception
   {
      super.setUp(  );
       //see if server should be run
       if (RUN_AXIS_SERVER) {
           startAxisServer(  );
       }
   }

   /**
    * Stops the embedded Axis server.
    *
    * @see TestCase#tearDown()
    */
   protected void tearDown(  )
   throws Exception
   {
      super.tearDown(  );
       //see if server was running before teardown
       if (RUN_AXIS_SERVER) {
           stopAxisServer(  );
       }
   }
}