/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util.timer;

import commonj.timers.CancelTimerListener;
import commonj.timers.Timer;
import commonj.timers.TimerListener;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.util.i18n.Keys;
import org.apache.ws.util.i18n.Messages;
import org.apache.ws.util.i18n.MessagesImpl;

/**
 * DOCUMENT_ME
 *
 * @author $author$
 * @version $Revision: 1.7 $
 */
public class TimerImpl
   implements Timer
{
   private static final Log LOG = LogFactory.getLog( TimerImpl.class );
    public static final Messages MSG = MessagesImpl.getInstance();
   private TimerTaskImpl timerTask;
   private TimerListener        listener;
   private long                 period;

   /**
    * Creates a new {@link TimerImpl} object.
    *
    * @param listener DOCUMENT_ME
    * @param period   DOCUMENT_ME
    */
   public TimerImpl( TimerListener listener,
                     long          period )
   {
      this.period      = period;
      this.listener    = listener;
   }

   /* (non-Javadoc)
    * @see commonj.timer.Timer#getPeriod()
    */
   public long getPeriod(  )
   {
      return this.period;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws IllegalStateException DOCUMENT_ME
    */
   public long getScheduledExecutionTime(  )
   throws IllegalStateException
   { //todo implement
      return 0;
   }

   /* (non-Javadoc)
    * @see commonj.timer.Timer#getTimerListener()
    */
   public TimerListener getTimerListener(  )
   {
      return this.listener;
   }

   /* (non-Javadoc)
    * @see commonj.timer.Timer#cancel()
    */
   public boolean cancel(  )
   {
      LOG.debug(MSG.getMessage( Keys.CANCELLING));

      if ( this.listener instanceof CancelTimerListener )
      {
         ( (CancelTimerListener) this.listener ).timerCancel( this );
      }

      return timerTask.cancel(  );
   }

   /* (non-Javadoc)
    * @see commonj.timer.Timer#scheduledExecutionTime()
    */
   public long scheduledExecutionTime(  )
   {
      return this.timerTask.scheduledExecutionTime(  ) + this.period;
   }

   /**
    * @param timerTask The timerTask to set.
    */
   protected void setTimerTask( TimerTaskImpl timerTask )
   {
      this.timerTask = timerTask;
   }

   /**
    * @return Returns the timerTask.
    */
   protected TimerTaskImpl getTimerTask(  )
   {
      return this.timerTask;
   }
}