/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util.timer;

import commonj.timers.Timer;
import commonj.timers.TimerListener;

import java.util.TimerTask;

/**
 * LOG-DONE
 * DOCUMENT_ME
 *
 * NOTE: Previously named TimerListenerWrapper.
 */
public class TimerTaskImpl
      extends TimerTask
{
   private Timer m_timer;
   private TimerListener m_timerListener;
   private TimerManagerImpl m_timerManager;
   private boolean m_suspended;
   private boolean m_expired;

   /**
    * DOCUMENT_ME
    *
    * @param timer
    * @param timerManager
    */
   public TimerTaskImpl( Timer timer,
                                TimerManagerImpl timerManager )
   {
      m_timer = timer;
      if ( timer instanceof TimerImpl )
      {
         ( (TimerImpl) timer ).setTimerTask( this );
      }
      m_timerListener = timer.getTimerListener();
      m_timerManager = timerManager;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean cancel()
   {
      m_timerManager.removeTask( this );
      return super.cancel();
   }

   /**
    * DOCUMENT_ME
    */
   public synchronized void resume()
   {
      m_suspended = false;
      if ( m_expired )
      {
         executeTimerExpiredAction();
         m_expired = false;
      }
   }

   /* (non-Javadoc)
    * @see java.lang.Runnable#run()
    */
   public synchronized void run()
   {
      if ( !m_suspended )
      {
         executeTimerExpiredAction();
      }
      else
      {
         m_expired = true;
      }
   }

   private void executeTimerExpiredAction()
   {
      m_timerListener.timerExpired( m_timer );
      if ( m_timer.getPeriod() == 0 )
      {
         m_timerManager.removeTask( this );
      }
   }

   /**
    * DOCUMENT_ME
    */
   public synchronized void suspend()
   {
      m_suspended = true;
   }

   /**
    * @return Returns the listener.
    */
   protected TimerListener getListener()
   {
      return m_timerListener;
   }

   /**
    * @return Returns the timer.
    */
   protected Timer getTimer()
   {
      return m_timer;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected boolean stop()
   {
      return super.cancel();
   }
}