/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.oasisOpen.docs.wsrf.x2005.x01.wsrfInterop20Draft03Wsdl;

import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.ResourceContextException;
import org.apache.ws.resource.ResourceException;
import org.apache.ws.resource.ResourceUnknownException;
import org.apache.ws.resource.properties.faults.ResourceUnknownFaultException;
import org.apache.ws.resource.properties.impl.XmlBeansResourceProperty;
import org.apache.ws.resource.properties.v1_2_draft05.impl.NamespaceVersionHolderImpl;
import org.oasisOpen.docs.wsrf.x2005.x01.wsrfInterop20Draft03.CreationResponseDocument;
import org.oasisOpen.docs.wsrf.x2005.x01.wsrfInterop20Draft03.JobStateType;
import org.oasisOpen.docs.wsrf.x2005.x01.wsrfInterop20Draft03.MimeMediaTypeType;
import org.oasisOpen.docs.wsrf.x2005.x01.wsrfInterop20Draft03.MimeMediaTypes;
import org.oasisOpen.docs.wsrf.x2005.x01.wsrfInterop20Draft03.SendDocumentRequestDocument;
import org.xmlsoap.schemas.ws.x2004.x08.addressing.EndpointReferenceType;

/**
 * This class should be generated ONCE (and not overwritten) to maintain user-added code.
 * If there is a change to the WSDL, then the generated implemented interfaces
 * (representing the "base" portTypes) will change, thus showing a compile error to the
 * user.
 * <p/>
 * NOTE: This class is generated. However, it will not be overwritten by subsequent
 * calls to the code generator.
 */
public class JobPortService
   extends AbstractJobPortService
   implements JobPortCustomOperationsPortType
{
   private ResourceContext m_resourceContext;

   /**
    * Creates a new {@link JobPortService } object.
    *
    * @param resourceContext DOCUMENT_ME
    */
   public JobPortService( ResourceContext resourceContext )
   {
      m_resourceContext = resourceContext;
      init(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public ResourceContext getResourceContext(  )
   {
      return m_resourceContext;
   }

   /**
    * DOCUMENT_ME
    *
    * @param requestDoc DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public org.oasisOpen.docs.wsrf.x2005.x01.wsrfInterop20Draft03.CreationResponseDocument Send_Document( org.oasisOpen.docs.wsrf.x2005.x01.wsrfInterop20Draft03.SendDocumentRequestDocument requestDoc )
   {
      CreationResponseDocument                        responseDoc         = createResponseDocument(  );
      CreationResponseDocument.CreationResponse       creationResponse    = responseDoc.getCreationResponse(  );
      SendDocumentRequestDocument.SendDocumentRequest sendDocumentRequest = requestDoc.getSendDocumentRequest(  );
      //DocumentContentType                             documentContent     =
         //sendDocumentRequest.getDocumentContent(  );
      String                                          documentFormat      =
         sendDocumentRequest.getDocumentFormat(  );
      boolean                                         lastDocument        =
         sendDocumentRequest.getLastDocument(  );
      JobPortResource                                       jobResource         = null;
      try
      {

          jobResource         =
                         (JobPortResource) getResourceContext(  ).getResource(  );

         if ( isDocTypeSupported( documentFormat, jobResource ) )
         {

             if (lastDocument)
             {
                 jobResource.enqueue();
             }

             addJobInfoToResponse(jobResource,creationResponse);
         }
         else
         {
            creationResponse.setSuccessStatusCode( "server-error-internal-error" );
         }
      }
      catch (ResourceUnknownException nsr)
      {
          try
          {
              throw new ResourceUnknownFaultException(new NamespaceVersionHolderImpl(),getResourceContext().getResourceKey(), getResourceContext().getServiceName() );
          }
          catch (ResourceContextException e)
          {
              throw new ClientErrorNotPossibleFaultException(new NamespaceVersionHolderImpl(), "Error Occurred: " + e.getLocalizedMessage());
          }
      }

      catch ( Exception e )
      {
         throw new ClientErrorNotPossibleFaultException(new NamespaceVersionHolderImpl(), "Error Occurred: " + e.getLocalizedMessage());
      }

      return responseDoc;
   }

   private void addJobInfoToResponse( JobPortResource                           instance,
                                      CreationResponseDocument.CreationResponse creationResponse )
   {
      org.apache.ws.resource.properties.ResourcePropertySet resourcePropertySet =
         instance.getResourcePropertySet(  );
      creationResponse.setJobReference( (EndpointReferenceType) ( (XmlBeansResourceProperty) resourcePropertySet
                                                                  .get( JobPortPropertyQNames.JOB_REFERENCE ) )
                                                                .toXmlObjects(  )[0] );
      JobStateType jobState =
         (JobStateType) ( (XmlBeansResourceProperty) resourcePropertySet.get( JobPortPropertyQNames.JOB_STATE ) )
                        .toXmlObjects(  )[0];
      creationResponse.xsetJobState( jobState ); //only way I could get to compile
      creationResponse.setSuccessStatusCode( "successful-ok" );
   }

   private boolean isDocTypeSupported(String documentFormat, JobPortResource jobResource)
   throws ResourceException
   {
      boolean                     docTypeSupported    = false;

      try
      {
         PrinterPortResource             printResource           = jobResource.getPrinterResource();
         XmlBeansResourceProperty        xmlPropDocFormat        =
            (XmlBeansResourceProperty) printResource.getResourcePropertySet(  ).get( PrinterPortPropertyQNames.DOCUMENT_FORMAT_SUPPORTED );
         MimeMediaTypes documentFormatSupported           =
            (MimeMediaTypes) xmlPropDocFormat.toXmlObjects(  )[0];

         MimeMediaTypeType.Enum[]        mimeMediaTypeArray      =
            documentFormatSupported.getMimeMediaTypeArray(  );

         for ( int i = 0; i < mimeMediaTypeArray.length; i++ )
         {
            MimeMediaTypeType.Enum enum = mimeMediaTypeArray[i];
            if ( enum.toString().equals( documentFormat ) )
            {
               docTypeSupported = true;
            }
         }
      }
      catch ( Exception e )
      {
         throw new ClientErrorNotPossibleFaultException(new NamespaceVersionHolderImpl(), "Error occurred when checking if doc type is supported: " + e);
      }

      return docTypeSupported;
   }

   private CreationResponseDocument createResponseDocument(  )
   {
      CreationResponseDocument creationResponseDocument = CreationResponseDocument.Factory.newInstance(  );
      creationResponseDocument.addNewCreationResponse(  );
      return creationResponseDocument;
   }
}