/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package example.filesystem.callback;

import example.filesystem.backend.FileSystem;
import org.apache.ws.resource.example.filesystem.OptionsDocument;
import org.apache.ws.resource.properties.ResourceProperty;
import org.apache.ws.resource.properties.SetResourcePropertyCallback;

import javax.xml.namespace.QName;
import java.util.Iterator;
import java.util.List;

/**
 * A callback for the Options resource property.
 */
public class OptionsCallback
        implements SetResourcePropertyCallback
{
    /**
     * DOCUMENT_ME
     */
    FileSystem m_fileSystem;

    /**
     * Creates a new {@link OptionsCallback} object.
     *
     * @param fileSystem DOCUMENT_ME
     */
    public OptionsCallback( FileSystem fileSystem )
    {
        m_fileSystem = fileSystem;
    }

    /**
     * DOCUMENT_ME
     *
     * @param propQName DOCUMENT_ME
     */
    public void deleteProperty( QName propQName )
    {
        return; // no need to implement - Apollo will never call delete for a prop whose minOccurs != 0
    }

    /**
     * DOCUMENT_ME
     *
     * @param prop DOCUMENT_ME
     */
    public void insertProperty( Object[] prop )
    {
        return; // no need to implement - Apollo will never call insert for a prop whose minOccurs == its maxOccurs
    }

    /**
     * DOCUMENT_ME
     *
     * @param prop DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public ResourceProperty refreshProperty( ResourceProperty prop )
    {
        Iterator iterator = prop.iterator();
        while ( iterator.hasNext() )
        {
            OptionsDocument.Options o = (OptionsDocument.Options) iterator.next();
            clearOptionsFromProperty( o );

            //add current options...
            List options = m_fileSystem.getOptions();
            for ( int i = 0; i < options.size(); i++ )
            {
                o.addOption( options.get( i ).toString() );
            }
        }

        return prop;
    }

    /**
     * DOCUMENT_ME
     *
     * @param prop DOCUMENT_ME
     */
    public void updateProperty( Object[] prop )
    {
        List backendOptions = m_fileSystem.getOptions();
        // Options prop has cardinality of 1, so passed array will always have exactly one element
        OptionsDocument.Options o = (OptionsDocument.Options) prop[0];
        String[] optionArray = o.getOptionArray();
        for ( int j = 0; j < optionArray.length; j++ )
        {
            backendOptions.add( optionArray[j] );
        }
    }

    private void clearOptionsFromProperty( OptionsDocument.Options o )
    {
        //remove the options...
        for ( int i = 0; i < o.sizeOfOptionArray(); i++ )
        {
            o.removeOption( i );
        }
    }
}