package org.apache.ws.resource.properties;

import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlInt;

import javax.xml.namespace.QName;

/**
 * @author Sal Campana
 */
public class SushiCallback implements SetResourcePropertyCallback
{
    private SushiPlate m_plate;
    private boolean m_deleteInvoked, m_insertInvoked, m_updateInvoked;

    public SushiCallback( SushiPlate plate )
    {
        m_plate = plate;
    }

    public SushiPlate getPlate()
    {
        return m_plate;
    }

    public ResourceProperty refreshProperty( ResourceProperty prop )
    {
        QName propName = getPropName( prop );

        if ( SushiPropertyQNames.EBI.equals( propName ) )
        {
            Ebi ebi = m_plate.getEbi();
            String pieces = ebi.getNumberOfPieces();
            try
            {
                XmlInt ebiElem = (XmlInt) prop.get( 0 );
                XmlBeanUtils.setValue( ebiElem, pieces );
            }
            catch ( Exception e )
            {
                e.printStackTrace();
            }
        }
        return prop;
    }

    private QName getPropName( ResourceProperty prop )
    {
        return prop.getMetaData().getName();
    }

    public void deleteProperty( QName propQName )
    {
        m_deleteInvoked = true;
    }

    public void insertProperty( Object[] prop )
    {
        m_insertInvoked = true;
    }

    public void updateProperty( Object[] prop )
    {
        m_updateInvoked = true;
        Object o = prop[0];
        if ( o instanceof XmlObject )
        {
            XmlObject xmlObject = (XmlObject) o;
            QName name = XmlBeanUtils.getName( xmlObject );
            if ( name.getLocalPart().equals( "Ebi" ) )
            {
                m_plate.getEbi().setNumberOfPieces( XmlBeanUtils.getValue( xmlObject ) );
            }
        }

    }

    public boolean deleteWasInvoked()
    {
        return m_deleteInvoked;
    }

    public boolean insertWasInvoked()
    {
        return m_insertInvoked;
    }

    public boolean updateWasInvoked()
    {
        return m_updateInvoked;
    }
}
