package org.apache.ws.resource.properties;

import org.apache.ws.resource.InvalidResourceKeyException;
import org.apache.ws.resource.Resource;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.ResourceContextException;
import org.apache.ws.resource.ResourceException;
import org.apache.ws.resource.ResourceHome;
import org.apache.ws.resource.ResourceKey;
import org.apache.ws.resource.faults.ResoureKeyHeaderNotFoundFaultException;
import org.apache.ws.resource.impl.SimpleTypeResourceKey;
import org.apache.ws.util.NameUtils;

import javax.xml.namespace.QName;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.SOAPMessage;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;

/**
 * @author Sal Campana
 */
public class SushiResourceContext extends ResourceContext
{
    ResourceHome m_home;
    private Resource m_resource = null;
    private SimpleTypeResourceKey m_key = null;
    public static final String ID_KEY = "1212";
    private String m_serviceName = "sushi";

    /**
     * Creates a new {@link SushiResourceContext} object.
     */
    public SushiResourceContext()
    {
       this( true );
    }

    /**
     * Creates a new {@link SushiResourceContext} object.
     */
    public SushiResourceContext( boolean isOpenContent )
    {
        m_home = new SushiHome( isOpenContent );
        try
        {
            ( (SushiHome) m_home ).getInstance( this );
            m_resource = m_home.find( this.m_key );
        }
        catch ( ResourceException e )
        {
            e.printStackTrace();
        }
        catch ( ResourceContextException e )
        {
            e.printStackTrace();
        }

    }

    /**
     * Creates a new {@link SushiResourceContext} object.
     *
     * @param pathToPropsXsd
     */
    public SushiResourceContext( String pathToPropsXsd )
    {

        try
        {
            ( (SushiHome) m_home ).setResourcePropsDocPath( pathToPropsXsd );
            ( (SushiHome) m_home ).getInstance( this );
            m_resource = m_home.find( this.m_key );
        }
        catch ( ResourceException e )
        {
            e.printStackTrace();
        }
        catch ( ResourceContextException e )
        {
            e.printStackTrace();
        }

    }

    public synchronized ResourceHome getResourceHome() throws ResourceContextException
    {
        return m_home;
    }

    public ResourceKey getResourceKey( QName keyName, Class keyClass ) throws ResourceContextException, ResoureKeyHeaderNotFoundFaultException
    {
        return m_key;
    }

    public SOAPHeaderElement getResourceKeyHeader() throws ResourceContextException
    {
        return (SOAPHeaderElement) m_key.toSOAPElement();
    }

    public SOAPHeaderElement getResourceKeyHeader( QName keyName ) throws ResourceContextException
    {
        return (SOAPHeaderElement) m_key.toSOAPElement();
    }

    public SOAPMessage getSOAPMessage()
    {
        try
        {
            return MessageFactory.newInstance().createMessage();
        }
        catch ( SOAPException e )
        {
            e.printStackTrace();
        }
        return null;
    }

    public ResourceKey getResourceKey() throws ResourceContextException
    {
        if ( m_key == null )
        {
            try
            {
                MessageFactory messageFactory = MessageFactory.newInstance();
                SOAPMessage message = messageFactory.createMessage();
                SOAPHeader soapHeader = message.getSOAPHeader();
                SOAPHeaderElement soapHeaderElement = soapHeader.addHeaderElement( NameUtils.toName( new QName( AbstractSushiService.TARGET_NSURI, "ResourceID", AbstractSushiService.TARGET_NSPREFIX ) ) );
                soapHeaderElement.addTextNode( ID_KEY );
                m_key = new SimpleTypeResourceKey( soapHeaderElement );
            }
            catch ( SOAPException e )
            {
                e.printStackTrace();
            }
            catch ( InvalidResourceKeyException e )
            {
                e.printStackTrace();
            }
        }
        return m_key;
    }

    public String getServiceName()
    {
        return m_serviceName;
    }

    public URL getServiceURL()
    {
        try
        {
            return new URL( "http://localhost:8080/muse/" + m_serviceName );
        }
        catch ( MalformedURLException murle )
        {
            murle.printStackTrace();
        }
        return null;
    }

    /**
     * Gets the base URL from which service urls are based.
     *
     * @return The String representation of the base url
     */
    public String getBaseURL()
    {
        return null;
    }

    public boolean containsProperty( String name )
    {
        return false;
    }

    public void removeProperty( String name )
    {

    }

    public void setProperty( String name, Object value )
    {

    }

    public Object getProperty( String name )
    {
        return null;
    }

    public Iterator getPropertyNames()
    {
        return null;
    }

    public Resource getResource() throws ResourceContextException, ResourceException
    {
        return m_resource;
    }

}
