/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.impl;

import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.apache.ws.resource.properties.AbstractResourcePropertiesTestCase;
import org.apache.ws.resource.properties.ResourceProperty;
import org.apache.ws.resource.properties.SushiPropertyQNames;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.xmlsoap.schemas.ws.x2003.x03.addressing.EndpointReferenceDocument;

import javax.xml.namespace.QName;
import java.util.Iterator;

/**
 * Test case for {@link XmlBeansResourcePropertySet}.
 *
 * @author Ian P. Springer
 */
public class XmlBeansResourcePropertySetTestCase
      extends AbstractResourcePropertiesTestCase
{

   private static final QName EPR_TYPE_QNAME = new QName( "http://schemas.xmlsoap.org/ws/2003/03/addressing", "EndpointReference", "wsa" );

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void setUp()
         throws Exception
   {
      initResourcePropsDoc();
   }

   /**
    * DOCUMENT_ME
    *
    * @param args DOCUMENT_ME
    */
   public static void main( String[] args )
   {
      TestResult result = TestRunner.run( XmlBeansResourcePropertySetTestCase.suite() );
      if ( !result.wasSuccessful() )
      {
         System.exit( 1 );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static Test suite()
   {
      return new TestSuite( XmlBeansResourcePropertySetTestCase.class );
   }

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void testAdd()
         throws Exception
   {
      XmlBeansResourcePropertyMetaData metaData =
            new XmlBeansResourcePropertyMetaData( EndpointReferenceDocument.type.getElementProperty( EPR_TYPE_QNAME ), false );
      m_resourcePropSet.add( metaData.create( m_resourcePropSet ) );
      ResourceProperty resourceProp = m_resourcePropSet.get( EPR_TYPE_QNAME );
      assertNotNull( resourceProp );
      assertEquals( EPR_TYPE_QNAME,
            resourceProp.getMetaData().getName() );
   }

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void testCreate()
         throws Exception
   {
      XmlBeansResourcePropertyMetaData metaData =
            new XmlBeansResourcePropertyMetaData( EndpointReferenceDocument.type.getElementProperty( EPR_TYPE_QNAME  ), false );
      ResourceProperty resourceProp = metaData.create( m_resourcePropSet );
      assertNotNull( resourceProp );
      assertEquals( EPR_TYPE_QNAME,
            resourceProp.getMetaData().getName() );
   }

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void testGet()
         throws Exception
   {
      ResourceProperty resourceProp = m_resourcePropSet.get( SushiPropertyQNames.AKAGI );
      assertEquals( 1, resourceProp.size() );
      assertEquals( SushiPropertyQNames.AKAGI, resourceProp.getMetaData().getName() );
      XmlObject propElemXBean = (XmlObject) resourceProp.get( 0 );
      assertEquals( "24", XmlBeanUtils.getValue( propElemXBean ) );
   }

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void testIterator()
         throws Exception
   {
      Iterator iter = m_resourcePropSet.iterator();
      int i = 0;
      while ( iter.hasNext() )
      {
         assertTrue( iter.next() instanceof ResourceProperty );
         i++;
      }
      assertEquals( 9, i );
   }

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void testRemove()
         throws Exception
   {
      ResourceProperty resourceProp = m_resourcePropSet.get( SushiPropertyQNames.IKA );
      assertEquals( 1, resourceProp.size() );
      m_resourcePropSet.remove( SushiPropertyQNames.IKA );
      resourceProp = m_resourcePropSet.get( SushiPropertyQNames.IKA );
      assertNull( resourceProp );
   }

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void testToElement()
         throws Exception
   {
      //TODO: Test goes here...
   }

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void testToSOAPElement()
         throws Exception
   {
      //TODO: Test goes here...
   }



}